# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from __future__ import annotations

from typing import Any
from typing_extensions import override

from ._proxy import LazyProxy


class ResourcesProxy(LazyProxy[Any]):
    """A proxy for the `google.genai._interactions.resources` module.

    This is used so that we can lazily import `google.genai._interactions.resources` only when
    needed *and* so that users can just import `google.genai._interactions` and reference `google.genai._interactions.resources`
    """

    @override
    def __load__(self) -> Any:
        import importlib

        mod = importlib.import_module("google.genai._interactions.resources")
        return mod


resources = ResourcesProxy().__as_proxied__()



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


"""Tests for generate_content_part."""

import base64
import os

from pydantic import ValidationError
import pytest

from ... import _transformers as t
from ... import errors
from ... import types
from .. import pytest_helper


IMAGE_PNG_FILE_PATH = os.path.abspath(
    os.path.join(os.path.dirname(__file__), '../data/google.png')
)
IMAGE_JPEG_FILE_PATH = os.path.abspath(
    os.path.join(os.path.dirname(__file__), '../data/google.jpg')
)
APPLICATION_PDF_FILE_PATH = os.path.abspath(
    os.path.join(os.path.dirname(__file__), '../data/story.pdf')
)
VIDEO_MP4_FILE_PATH = os.path.abspath(
    os.path.join(os.path.dirname(__file__), '../data/animal.mp4')
)
AUDIO_MP3_FILE_PATH = os.path.abspath(
    os.path.join(os.path.dirname(__file__), '../data/pixel.m4a')
)
with open(IMAGE_PNG_FILE_PATH, 'rb') as image_file:
  image_bytes = image_file.read()
  image_string = base64.b64encode(image_bytes).decode('utf-8')
with open(APPLICATION_PDF_FILE_PATH, 'rb') as pdf_file:
  pdf_bytes = pdf_file.read()
with open(VIDEO_MP4_FILE_PATH, 'rb') as video_file:
  video_bytes = video_file.read()
with open(AUDIO_MP3_FILE_PATH, 'rb') as audio_file:
  audio_bytes = audio_file.read()

test_table: list[pytest_helper.TestTableItem] = [
    pytest_helper.TestTableItem(
        name='test_image_uri',
        parameters=types._GenerateContentParameters(
            model='gemini-2.5-flash',
            contents=[
                t.t_content('What is this image about?'),
                t.t_content(
                    {
                        'role': 'user',
                        'parts': [
                            types.PartDict({
                                'file_data': {
                                    'file_uri': (
                                        'gs://generativeai-downloads/images/scones.jpg'
                                    ),
                                    'mime_type': 'image/jpeg',
                                }
                            })
                        ],
                    },
                ),
            ],
        ),
        exception_if_mldev='400',
    ),
    pytest_helper.TestTableItem(
        name='test_external_file_uri',
        parameters=types._GenerateContentParameters(
            model='gemini-2.5-flash',
            contents=[
                t.t_content('What is this image about?'),
                t.t_content(
                    {
                        'role': 'user',
                        'parts': [
                            types.PartDict({
                                'file_data': {
                                    'file_uri': (
                                        'https://storage.googleapis.com/cloud-samples-data/generative-ai/image/scones.jpg'
                                    ),
                                    'mime_type': 'image/jpeg',
                                }
                            })
                        ],
                    },
                ),
            ],
        ),
        exception_if_mldev='400',
    ),
    pytest_helper.TestTableItem(
        name='test_image_png_file_uri',
        skip_in_api_mode=(
            'Name of the file is hardcoded, only supporting replay mode.'
        ),
        parameters=types._GenerateContentParameters(
            model='gemini-2.5-flash',
            contents=[
                t.t_content('What is this image about?'),
                t.t_content(
                    {
                        'role': 'user',
                        'parts': [
                            types.PartDict({
                                'file_data': {
                                    'file_uri': (
                                        'https://generativelanguage.googleapis.com/v1beta/files/864z7ft4m14h'
                                    ),
                                    'mime_type': 'image/png',
                                }
                            })
                        ],
                    },
                ),
            ],
        ),
        exception_if_vertex='403',
    ),
    pytest_helper.TestTableItem(
        name='test_image_jpg_file_uri',
        skip_in_api_mode=(
            'Name of the file is hardcoded, only supporting replay mode.'
        ),
        parameters=types._GenerateContentParameters(
            model='gemini-2.5-flash',
            contents=[
                t.t_content('What is this image about?'),
                t.t_content(
                    {
                        'role': 'user',
                        'parts': [
                            types.PartDict({
                                'file_data': {
                                    'file_uri': (
                                        'https://generativelanguage.googleapis.com/v1beta/files/22xab9a8jp0v'
                                    ),
                                    'mime_type': 'image/jpeg',
                                }
                            })
                        ],
                    },
                ),
            ],
        ),
        exception_if_vertex='403',
    ),
    pytest_helper.TestTableItem(
        name='test_image_inline_part_media_resolution',
        parameters=types._GenerateContentParameters(
            model='gemini-2.5-flash',
            contents=[
                t.t_content('What is this image about?'),
                t.t_content(
                    {
                        'role': 'user',
                        'parts': [
                            types.Part(
                                inline_data=types.Blob(
                                    data=image_string, mime_type='image/png'
                                ),
                                media_resolution={'level': 'MEDIA_RESOLUTION_LOW'}
                            )
                        ],
                    },
                ),
            ],
        ),
    ),
    pytest_helper.TestTableItem(
        name='test_application_pdf_file_uri',
        skip_in_api_mode=(
            'Name of the file is hardcoded, only supporting replay mode.'
        ),
        parameters=types._GenerateContentParameters(
            model='gemini-2.5-flash',
            contents=[
                t.t_content(
                    'Summarize the pdf in concise and professional tone.'
                ),
                t.t_content(
                    {
                        'role': 'user',
                        'parts': [
                            types.PartDict({
                                'file_data': {
                                    'file_uri': (
                                        'https://generativelanguage.googleapis.com/v1beta/files/9tnpjy0re13a'
                                    ),
                                    'mime_type': 'application/pdf',
                                }
                            })
                        ],
                    },
                ),
            ],
        ),
        exception_if_vertex='403',
    ),
    pytest_helper.TestTableItem(
        name='test_video_mp4_file_uri',
        skip_in_api_mode=(
            'Name of the file is hardcoded, only supporting replay mode.'
        ),
        parameters=types._GenerateContentParameters(
            model='gemini-2.5-flash',
            contents=[
                t.t_content(
                    """
                    summarize the video in concise and professional tone.
                    the summary should include all important information said in the video.
                    """,
                ),
                t.t_content(
                    {
                        'role': 'user',
                        'parts': [
                            types.PartDict({
                                'file_data': {
                                    'file_uri': (
                                        'https://generativelanguage.googleapis.com/v1beta/files/i2ojhd8z99uk'
                                    ),
                                    'mime_type': 'video/mp4',
                                }
                            })
                        ],
                    },
                ),
            ],
        ),
        exception_if_vertex='403',
    ),
    pytest_helper.TestTableItem(
        name='test_audio_m4a_file_uri',
        skip_in_api_mode=(
            'Name of the file is hardcoded, only supporting replay mode.'
        ),
        parameters=types._GenerateContentParameters(
            model='gemini-2.5-flash',
            contents=[
                t.t_content(
                    """
                    Provide a summary for the audio in the beginning of the transcript.
                    Provide concise chapter titles with timestamps.
                    Do not make up any information that is not part of the audio.
                """,
                ),
                t.t_content(
                    {
                        'role': 'user',
                        'parts': [
                            types.PartDict({
                                'file_data': {
                                    'file_uri': (
                                        'https://generativelanguage.googleapis.com/v1beta/files/0o9iyit198g1'
                                    ),
                                    'mime_type': 'audio/mp4',
                                }
                            })
                        ],
                    },
                ),
            ],
        ),
        exception_if_vertex='403',
    ),
    pytest_helper.TestTableItem(
        name='test_mldev_video_offset_and_fps',
        skip_in_api_mode=(
            'Name of the file is hardcoded, only supporting replay mode.'
        ),
        parameters=types._GenerateContentParameters(
            model='gemini-2.5-flash',
            contents=[
                types.Content(
                    role='user',
                    parts=[
                        types.Part(text='summarize this video'),
                        types.Part(
                            file_data=types.FileData(
                                file_uri='https://generativelanguage.googleapis.com/v1beta/files/fq9r5ftyqrho',
                                mime_type= 'video/mp4',
                            ),
                            video_metadata=types.VideoMetadata(
                                    start_offset='0s',
                                    end_offset= '5s',
                                    fps= 3,
                            )
                        )
                    ]
                ),
            ],
        ),
        exception_if_vertex='403',
    ),
    pytest_helper.TestTableItem(
        name='test_video_gcs_file_uri',
        skip_in_api_mode=(
            'Name of the file is hardcoded, only supporting replay mode.'
        ),
        parameters=types._GenerateContentParameters(
            model='gemini-2.5-flash',
            contents=[
                t.t_content('what is the video about?'),
                t.t_content(
                    {
                        'role': 'user',
                        'parts': [
                            types.PartDict({
                                'file_data': {
                                    'file_uri': (
                                        'gs://generativeai-downloads/videos/Big_Buck_Bunny.mp4'
                                    ),
                                    'mime_type': 'video/mp4',
                                },
                                'video_metadata': {
                                    'start_offset': '0s',
                                    'end_offset': '10s',
                                    'fps': 3,
                                },
                            })
                        ],
                    },
                ),
            ],
        ),
        exception_if_mldev='400',
    ),
    pytest_helper.TestTableItem(
        name='test_image_base64',
        parameters=types._GenerateContentParameters(
            model='gemini-2.5-flash',
            contents=[
                t.t_content('What is this image about?'),
                t.t_content(
                    {
                        'role': 'user',
                        'parts': [
                            types.Part(
                                inline_data=types.Blob(
                                    data=image_string, mime_type='image/png'
                                )
                            )
                        ],
                    },
                ),
            ],
        ),
    ),
    pytest_helper.TestTableItem(
        name='test_union_none_part',
        parameters=types._GenerateContentParameters(
            model='gemini-2.5-flash',
            contents=[],
        ),
        exception_if_mldev='contents',
        exception_if_vertex='contents',
        has_union=True,
    ),
    pytest_helper.TestTableItem(
        name='test_dict_content',
        parameters=types._GenerateContentParameters(
            model='gemini-2.5-flash',
            contents=t.t_contents(
                types.ContentDict(
                    {'role': 'user', 'parts': [{'text': 'what is your name?'}]}
                ),
            ),
        ),
    ),
    pytest_helper.TestTableItem(
        name='test_union_part_list',
        parameters=types._GenerateContentParameters(
            model='gemini-2.5-flash',
            contents=['What is your name?'],
        ),
        has_union=True,
    ),
]
pytestmark = pytest_helper.setup(
    file=__file__,
    globals_for_file=globals(),
    test_method='models.generate_content',
    test_table=test_table,
)
pytest_plugins = ('pytest_asyncio',)


def test_empty_part(client):
  client.models.generate_content(
      model='gemini-2.5-flash',
      contents=t.t_contents(['']),
  )


def test_none_list_part(client):
  # pydantic will raise ValidationError
  with pytest.raises(ValidationError):
    client.models.generate_content(
        model='gemini-2.5-flash',
        contents=[None],
    )


def test_image_file(client):
  client.models.generate_content(
      model='gemini-2.5-flash',
      contents=[
          'What is this image about?',
          {'inline_data': {'data': image_bytes, 'mimeType': 'image/png'}},
      ],
  )


def test_image_jpeg(client, image_jpeg):
  client.models.generate_content(
      model='gemini-2.5-flash',
      contents=['What is this image about?', image_jpeg],
  )


def test_from_uri(client):
  # gs://generativeai-downloads/images/scones.jpg isn't supported in MLDev
  with pytest_helper.exception_if_mldev(client, errors.ClientError):
    client.models.generate_content(
        model='gemini-2.5-flash',
        contents=[
            'What is this image about?',
            types.Part.from_uri(
                file_uri='gs://generativeai-downloads/images/scones.jpg',
                mime_type='image/jpeg',
            ),
        ],
    )


def test_user_content_text(client):
  response = client.models.generate_content(
      model='gemini-2.5-flash',
      contents=types.UserContent(parts='why is the sky blue?'),
  )
  assert response.text


def test_user_content_part(client):
  with pytest_helper.exception_if_mldev(client, errors.ClientError):
    response = client.models.generate_content(
        model='gemini-2.5-flash',
        contents=types.UserContent(
            parts=[
                'what is this image about?',
                types.Part.from_uri(
                    file_uri='gs://generativeai-downloads/images/scones.jpg',
                    mime_type='image/jpeg',
                ),
            ]
        ),
    )
    assert response.text


def test_model_content_text(client):
  with pytest_helper.exception_if_mldev(client, errors.ClientError):
    response = client.models.generate_content(
        model='gemini-2.5-flash',
        contents=[
            types.UserContent(
                parts=[
                    'what is this image about?',
                    types.Part.from_uri(
                        file_uri=(
                            'gs://generativeai-downloads/images/scones.jpg'
                        ),
                        mime_type='image/jpeg',
                    ),
                ]
            ),
            types.ModelContent(
                parts=(
                    'The image is about a cozy breakfast or brunch with'
                    ' blueberry scones, coffee, and fresh flowers.'
                )
            ),
            types.UserContent(
                parts='Is this a good environment for a family gathering?'
            ),
        ],
    )
    assert response.text


def test_from_file_input(client):
  with pytest_helper.exception_if_vertex(client, ValueError):
    file = client.files.upload(file='tests/data/story.txt')
    client.models.generate_content(
        model='gemini-2.5-flash',
        contents=file,
    )
    client.models.generate_content(
        model='gemini-2.5-flash',
        contents=[
            'Summarize this file',
            file,
        ],
    )
    client.models.generate_content(
        model='gemini-2.5-flash',
        contents=[['Summarize this file', file]],
    )


def test_from_file_dict_input(client):
  with pytest_helper.exception_if_vertex(client, ValueError):
    file = client.files.upload(file='tests/data/story.txt')
    file_dict = file.model_dump()
    client.models.generate_content(
        model='gemini-2.5-flash',
        contents=file_dict,
    )
    client.models.generate_content(
        model='gemini-2.5-flash',
        contents=[
            'Summarize this file',
            file_dict,
        ],
    )
    client.models.generate_content(
        model='gemini-2.5-flash',
        contents=[['Summarize this file', file_dict]],
    )


def test_from_uploaded_file_uri(client):
  with pytest_helper.exception_if_vertex(client, ValueError):
    file = client.files.upload(file='tests/data/story.txt')
    file_part = types.Part.from_uri(file_uri=file.uri, mime_type='text/plain')
    client.models.generate_content(
        model='gemini-2.5-flash',
        contents=file_part,
    )
    client.models.generate_content(
        model='gemini-2.5-flash',
        contents=[
            'Summarize this file',
            file_part,
        ],
    )
    client.models.generate_content(
        model='gemini-2.5-flash',
        contents=[['Summarize this file', file_part]],
    )


def test_from_uri_inferred_mime_type(client):
  # gs://generativeai-downloads/images/scones.jpg isn't supported in MLDev
  with pytest_helper.exception_if_mldev(client, errors.ClientError):
    client.models.generate_content(
        model='gemini-2.5-flash',
        contents=[
            'What is this image about?',
            types.Part.from_uri(
                file_uri='gs://generativeai-downloads/images/scones.jpg'
            ),
        ],
    )


def test_from_uri_invalid_inferred_mime_type(client):
  # Throws ValueError if mime_type cannot be inferred.
  with pytest.raises(ValueError):
    client.models.generate_content(
        model='gemini-2.5-flash',
        contents=[
            'What is this image about?',
            types.Part.from_uri(
                file_uri='uri/without/mime/type'
            ),
        ],
    )


def test_audio_uri(client):
  with pytest_helper.exception_if_mldev(client, errors.ClientError):
    client.models.generate_content(
        model='gemini-2.5-flash',
        contents=[
            """
        Provide a summary for the audio in the beginning of the transcript.
        Provide concise chapter titles with timestamps.
        Do not make up any information that is not part of the audio.
        """,
            types.Part.from_uri(
                file_uri='gs://cloud-samples-data/generative-ai/audio/pixel.mp3',
                mime_type='audio/mpeg',
            ),
        ],
        config={
            'system_instruction': (
                'You are a helpful assistant for audio transcription.'
            )
        },
    )


def test_pdf_uri(client):
  with pytest_helper.exception_if_mldev(client, errors.ClientError):
    client.models.generate_content(
        model='gemini-2.5-flash',
        contents=[
            'summarize the pdf in concise and professional tone',
            types.Part.from_uri(
                file_uri='gs://cloud-samples-data/generative-ai/pdf/2403.05530.pdf',
                mime_type='application/pdf',
            ),
        ],
        config={
            'system_instruction': (
                'You are a helpful assistant for academic literature review.'
            )
        },
    )


def test_video_uri(client):
  with pytest_helper.exception_if_mldev(client, errors.ClientError):
    client.models.generate_content(
        model='gemini-2.5-flash',
        contents=[
            """
            summarize the video in concise and professional tone.
            the summary should include all important information said in the video.
            """,
            types.Part.from_uri(
                file_uri='gs://cloud-samples-data/generative-ai/video/pixel8.mp4',
                mime_type='video/mp4',
            ),
        ],
        config={
            'system_instruction': (
                'you are a helpful assistant for market research.'
            )
        },
    )


def test_video_audio_uri(client):
  with pytest_helper.exception_if_mldev(client, errors.ClientError):
    client.models.generate_content(
        model='gemini-2.5-flash',
        contents=[
            """
            Is the audio related to the video?
            If so, how?
            What are the common themes?
            What are the different emphases?
            """,
            types.Part.from_uri(
                file_uri='gs://cloud-samples-data/generative-ai/video/pixel8.mp4',
                mime_type='video/mp4',
            ),
            types.Part.from_uri(
                file_uri='gs://cloud-samples-data/generative-ai/audio/pixel.mp3',
                mime_type='audio/mpeg',
            ),
        ],
        config={
            'system_instruction': (
                'you are a helpful assistant for people with visual and hearing'
                ' disabilities.'
            )
        },
    )


def test_file(client):
  with pytest_helper.exception_if_vertex(client, errors.ClientError):
    file = types.File(
        uri='https://generativelanguage.googleapis.com/v1beta/files/8q6j6weg80ey',
        mime_type='text/plain',
    )
    client.models.generate_content(
        model='gemini-2.5-flash',
        contents=[
            'Summarize this file',
            file,
        ],
    )


def test_file_error(client):
  # missing mime_type
  with pytest.raises(ValueError):
    file = types.File(
        uri='https://generativelanguage.googleapis.com/v1beta/files/8q6j6weg80ey',
    )
    client.models.generate_content(
        model='gemini-2.5-flash',
        contents=[
            'Summarize this file',
            file,
        ],
    )


def test_from_text(client):
  client.models.generate_content(
      model='gemini-2.5-flash',
      contents=[types.Part.from_text(text='What is your name?')],
  )


def test_from_bytes_image(client):
  client.models.generate_content(
      model='gemini-2.5-flash',
      contents=[
          'What is this image about?',
          types.Part.from_bytes(data=image_bytes, mime_type='image/png'),
      ],
  )


def test_from_bytes_image_dict(client):
  client.models.generate_content(
      model='gemini-2.5-flash',
      contents=[
          {'text': 'What is this image about?'},
          {'inline_data': {'data': image_bytes, 'mimeType': 'image/png'}},
      ],
  )


def test_from_bytes_image_none(client):
  with pytest.raises(errors.ClientError) as e:
    client.models.generate_content(
        model='gemini-2.5-flash',
        contents=[
            {'text': 'What is this image about?'},
            {'inline_data': {'data': None, 'mimeType': 'image/png'}},
        ],
    )
  assert 'INVALID_ARGUMENT' in str(e)


def test_from_bytes_video(client):
  client.models.generate_content(
      model='gemini-2.5-flash',
      contents=[
          'What is this video about?',
          types.Part.from_bytes(data=video_bytes, mime_type='video/mp4'),
      ],
  )


def test_from_bytes_audio(client):
  client.models.generate_content(
      model='gemini-2.5-flash',
      contents=[
          'What is this audio about?',
          types.Part.from_bytes(data=audio_bytes, mime_type='audio/mpeg'),
      ],
  )


def test_from_bytes_pdf(client):
  client.models.generate_content(
      model='gemini-2.5-flash',
      contents=[
          'What is this pdf about?',
          types.Part.from_bytes(data=pdf_bytes, mime_type='application/pdf'),
      ],
  )


def test_from_function_call_response(client):
  function_call = types.Part.from_function_call(
      name='get_weather', args={'location': 'Boston'}
  )
  function_response = types.Part.from_function_response(
      name='get_weather', response={'weather': 'sunny'}
  )
  response = client.models.generate_content(
      model='gemini-2.5-flash',
      contents=[
          'what is the weather in Boston?',
          function_call,
          function_response,
      ],
  )

  assert 'sunny' in response.text
  assert 'Boston' in response.text


@pytest.mark.asyncio
async def test_image_base64_stream_async(client):
  async for part in await client.aio.models.generate_content_stream(
      model='gemini-2.5-flash',
      contents=[
          'What is this image about?',
          {'inline_data': {'data': image_string, 'mimeType': 'image/png'}},
      ],
  ):
    pass


# function_call and function_response are tested in generate_content_tools.py



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


from unittest.mock import patch


@patch.dict('sys.modules', {'PIL': None, 'PIL.Image': None})
def test_without_pil_installed_mocked():
  from ... import types

  type_names_in_union = [
      arg.__name__ if hasattr(arg, '__name__') else str(arg)
      for arg in types.PartUnion.__args__]
  assert 'Image' not in type_names_in_union

def test_without_pil_installed_mocked():
  from ... import types

  type_names_in_union = [
      arg.__name__ if hasattr(arg, '__name__') else str(arg)
      for arg in types.PartUnion.__args__]

  assert 'Image' in type_names_in_union


# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import pytest
from ... import types
from .. import pytest_helper


def get_current_weather(location: str) -> str:
  """Returns the current weather.

  Args:
    location: The location of a city and state, e.g. "San Francisco, CA".
  """
  return 'windy'


pytestmark = pytest_helper.setup(
    file=__file__,
    globals_for_file=globals(),
    test_method='models.generate_content',
)
pytest_plugins = ('pytest_asyncio',)


def test_generate_content_stream_with_function_and_thought_summaries(client):
  """Test when function tools are provided and thought summaries are enabled.

  Expected to answer weather based on function response.
  """
  config = types.GenerateContentConfig(
      tools=[get_current_weather],
      thinking_config=types.ThinkingConfig(include_thoughts=True),
  )
  stream = client.models.generate_content_stream(
      model='gemini-2.5-flash',
      contents='what is the weather in San Francisco, CA?',
      config=config,
  )

  chunk = None
  for chunk in stream:
    assert chunk is not None


@pytest.mark.asyncio
async def test_generate_content_stream_with_function_and_thought_summaries_async(
    client,
):
  """Test when function tools are provided and thought summaries are enabled.

  Expected to answer weather based on function response.
  """
  config = types.GenerateContentConfig(
      tools=[get_current_weather],
      thinking_config=types.ThinkingConfig(include_thoughts=True),
  )
  stream = await client.aio.models.generate_content_stream(
      model='gemini-2.5-flash',
      contents='what is the weather in San Francisco, CA?',
      config=config,
  )

  chunk = None
  async for chunk in stream:
    assert chunk is not None



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""Table tests for chats.send_message_stream()."""

from ... import pytest_helper
from pydantic import BaseModel


class _SendMessageStreamParameters(BaseModel):
    model: str
    message: str


def send_message_stream(client, parameters):
  chat = client.chats.create(model=parameters.model)
  response = chat.send_message_stream(parameters.message)
  for _ in response:
    pass

test_table: list[pytest_helper.TestTableItem] = [
    pytest_helper.TestTableItem(
        name="test_send_message_stream",
        parameters=_SendMessageStreamParameters(
            model="gemini-2.5-flash",
            message="Tell a joke.",
        ),
    ),
]

pytestmark = pytest_helper.setup(
    file=__file__,
    globals_for_file=globals(),
    test_method="send_message_stream",
    test_table=test_table,
)

pytest_plugins = ("pytest_asyncio",)



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from ... import _mcp_utils
from ... import types

try:
  from mcp import types as mcp_types
except ImportError as e:
  import sys

  if sys.version_info < (3, 10):
    raise ImportError(
        'MCP Tool requires Python 3.10 or above. Please upgrade your Python'
        ' version.'
    ) from e
  else:
    raise e


def test_empty_mcp_tools_list():
  """Test conversion of empty MCP tools list to Gemini tools list."""
  result = _mcp_utils.mcp_to_gemini_tools([])

  assert result == []


def test_unknown_field_conversion():
  """Test conversion of MCP tools with unknown fields to Gemini tools."""
  mcp_tools = [
      mcp_types.Tool(
          name='tool',
          description='tool-description',
          inputSchema={
              'type': 'object',
              'properties': {},
              'unknown_field': 'unknownField',
              'unknown_object': {},
          },
      ),
  ]
  result = _mcp_utils.mcp_to_gemini_tools(mcp_tools)
  assert result == [
      types.Tool(
          function_declarations=[
              types.FunctionDeclaration(
                  name='tool',
                  description='tool-description',
                  parameters=types.Schema(
                      type='OBJECT',
                      properties={},
                  ),
              ),
          ],
      ),
  ]


def test_items_conversion():
  """Test conversion of MCP tools with items to Gemini tools."""
  mcp_tools = [
      mcp_types.Tool(
          name='tool',
          description='tool-description',
          inputSchema={
              'type': 'array',
              'items': {
                  'type': 'object',
                  'properties': {
                      'key1': {
                          'type': 'string',
                      },
                      'key2': {
                          'type': 'number',
                      },
                  },
              },
          },
      ),
  ]
  result = _mcp_utils.mcp_to_gemini_tools(mcp_tools)
  assert result == [
      types.Tool(
          function_declarations=[
              types.FunctionDeclaration(
                  name='tool',
                  description='tool-description',
                  parameters=types.Schema(
                      type='ARRAY',
                      items=types.Schema(
                          type='OBJECT',
                          properties={
                              'key1': types.Schema(type='STRING'),
                              'key2': types.Schema(type='NUMBER'),
                          },
                      ),
                  ),
              ),
          ],
      ),
  ]


def test_any_of_conversion():
  """Test conversion of MCP tools with any_of to Gemini tools."""
  mcp_tools = [
      mcp_types.Tool(
          name='tool',
          description='tool-description',
          inputSchema={
              'type': 'object',
              'any_of': [
                  {
                      'type': 'string',
                  },
                  {
                      'type': 'number',
                  },
              ],
          },
      ),
  ]
  result = _mcp_utils.mcp_to_gemini_tools(mcp_tools)
  assert result == [
      types.Tool(
          function_declarations=[
              types.FunctionDeclaration(
                  name='tool',
                  description='tool-description',
                  parameters=types.Schema(
                      type='OBJECT',
                      any_of=[
                          types.Schema(type='STRING'),
                          types.Schema(type='NUMBER'),
                      ],
                  ),
              ),
          ],
      ),
  ]


def test_properties_conversion():
  """Test conversion of MCP tools with properties to Gemini tools."""
  mcp_tools = [
      mcp_types.Tool(
          name='tool',
          description='tool-description',
          inputSchema={
              'type': 'object',
              'properties': {
                  'key1': {
                      'type': 'string',
                  },
                  'key2': {
                      'type': 'number',
                  },
              },
          },
      ),
  ]
  result = _mcp_utils.mcp_to_gemini_tools(mcp_tools)
  assert result == [
      types.Tool(
          function_declarations=[
              types.FunctionDeclaration(
                  name='tool',
                  description='tool-description',
                  parameters=types.Schema(
                      type='OBJECT',
                      properties={
                          'key1': types.Schema(type='STRING'),
                          'key2': types.Schema(type='NUMBER'),
                      },
                  ),
              ),
          ],
      ),
  ]



"""Tests t_bytes methods in the _transformers module."""

import base64

from ... import _base_transformers as t

_RAW_BYTES = (
    b'\xfb\xf6\x9bq\xd7\x9f\x82\x18\xa3\x92Y\xa7\xa2\x9a\xab\xb2\xdb\xaf\xc3\x1c\xb3\x00\x10\x83\x10Q\x87'
    b' \x92\x8b0\xd3\x8fA\x14\x93QU\x97a\x9d5\xdb~9\xeb\xbf='
)


def test_t_bytes():
  assert t.t_bytes(_RAW_BYTES) == base64.b64encode(_RAW_BYTES).decode('ascii')
  assert t.t_bytes('string') == 'string'



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["GoogleMapsResult", "Place", "PlaceReviewSnippet"]


class PlaceReviewSnippet(BaseModel):
    """
    Encapsulates a snippet of a user review that answers a question about
    the features of a specific place in Google Maps.
    """

    review_id: Optional[str] = None
    """The ID of the review snippet."""

    title: Optional[str] = None
    """Title of the review."""

    url: Optional[str] = None
    """A link that corresponds to the user review on Google Maps."""


class Place(BaseModel):
    name: Optional[str] = None
    """Title of the place."""

    place_id: Optional[str] = None
    """The ID of the place, in `places/{place_id}` format."""

    review_snippets: Optional[List[PlaceReviewSnippet]] = None
    """
    Snippets of reviews that are used to generate answers about the
    features of a given place in Google Maps.
    """

    url: Optional[str] = None
    """URI reference of the place."""


class GoogleMapsResult(BaseModel):
    """The result of the Google Maps."""

    places: Optional[List[Place]] = None
    """The places that were found."""

    widget_context_token: Optional[str] = None
    """Resource name of the Google Maps widget context token."""



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._types import Base64FileInput
from .._utils import PropertyInfo
from .._models import set_pydantic_config

__all__ = ["FileSearchResultContentParam", "Result"]


class Result(TypedDict, total=False):
    """The result of the File Search."""

    custom_metadata: Iterable[object]
    """User provided metadata about the FileSearchResult."""


class FileSearchResultContentParam(TypedDict, total=False):
    """File Search result content."""

    call_id: Required[str]
    """Required. ID to match the ID from the function call block."""

    result: Required[Iterable[Result]]
    """Required. The results of the File Search."""

    type: Required[Literal["file_search_result"]]

    signature: Annotated[Union[str, Base64FileInput], PropertyInfo(format="base64")]
    """A signature hash for backend validation."""


set_pydantic_config(FileSearchResultContentParam, {"arbitrary_types_allowed": True})



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


"""Tests for models.get."""

import pytest
from ... import errors
from ... import types
from .. import pytest_helper


test_http_options = {'api_version': 'v1', 'headers': {'test': 'headers'}}

test_table: list[pytest_helper.TestTableItem] = [
    pytest_helper.TestTableItem(
        name='test_get_vertex_tuned_model',
        parameters=types._GetModelParameters(
            model='models/2171259487439028224'
        ),
        exception_if_mldev='404',
    ),
    pytest_helper.TestTableItem(
        name='test_get_mldev_tuned_model',
        parameters=types._GetModelParameters(
            model='tunedModels/generatenum5443-ekrw7ie9wis23zbeogbw6jq8'
        ),
        exception_if_vertex='404',
    ),
    pytest_helper.TestTableItem(
        name='test_get_vertex_tuned_model_with_http_options_in_method',
        parameters=types._GetModelParameters(
            model='models/2171259487439028224',
            config={
                'http_options': test_http_options,
            },
        ),
        exception_if_mldev='404',
    ),
    pytest_helper.TestTableItem(
        name='test_get_mldev_base_model_with_http_options_in_method',
        parameters=types._GetModelParameters(
            model='gemini-2.5-flash',
            config={
                'http_options': test_http_options,
            },
        ),
    ),
    pytest_helper.TestTableItem(
        name='test_get_base_model',
        parameters=types._GetModelParameters(model='gemini-2.5-flash'),
    ),
    pytest_helper.TestTableItem(
        name='test_get_base_model_with_models_prefix',
        parameters=types._GetModelParameters(model='models/gemini-2.5-flash'),
        exception_if_vertex='400',
    ),
]
pytestmark = pytest_helper.setup(
    file=__file__,
    globals_for_file=globals(),
    test_method='models.get',
    test_table=test_table,
)


@pytest.mark.asyncio
async def test_async_get_tuned_model(client):
  if client._api_client.vertexai:
    with pytest.raises(errors.ClientError) as e:
      await client.aio.models.get(model='tunedModels/generate-num-1896')
    assert '404' in str(e)
  else:
    response = await client.aio.models.get(
        model='tunedModels/generate-num-1896'
    )


@pytest.mark.asyncio
async def test_async_get_model(client):
  if client._api_client.vertexai:
    response = await client.aio.models.get(
        model='models/7687416965014487040',
        config={'http_options': test_http_options},
    )
  else:
    with pytest.raises(errors.ClientError) as e:
      await client.aio.models.get(
          model='models/7687416965014487040',
          config={'http_options': test_http_options},
      )
    assert '404' in str(e)



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""Error classes for the GenAI SDK."""

from typing import Any, Callable, Optional, TYPE_CHECKING, Union
import httpx
import json
import requests
from . import _common


if TYPE_CHECKING:
  from .replay_api_client import ReplayResponse
  import aiohttp
  from google.auth.aio.transport.aiohttp import Response as AsyncAuthorizedSessionResponse


class APIError(Exception):
  """General errors raised by the GenAI API."""
  code: int
  response: Union[
      requests.Response,
      'ReplayResponse',
      httpx.Response,
      'AsyncAuthorizedSessionResponse',
  ]

  status: Optional[str] = None
  message: Optional[str] = None

  def __init__(
      self,
      code: int,
      response_json: Any,
      response: Optional[
          Union[
              requests.Response,
              'ReplayResponse',
              httpx.Response,
              'AsyncAuthorizedSessionResponse',
          ]
      ] = None,
  ):
    if isinstance(response_json, list) and len(response_json) == 1:
      response_json = response_json[0]

    self.response = response
    self.details = response_json
    self.message = self._get_message(response_json)
    self.status = self._get_status(response_json)
    self.code = code if code else self._get_code(response_json)

    super().__init__(f'{self.code} {self.status}. {self.details}')

  def __reduce__(
      self,
  ) -> tuple[Callable[..., 'APIError'], tuple[dict[str, Any]]]:
    """Returns a tuple that can be used to reconstruct the error for pickling."""
    state = self.__dict__.copy()
    return (self.__class__._rebuild, (state,))

  @staticmethod
  def _rebuild(state: dict[str, Any]) -> 'APIError':
    """Rebuilds the error from the state."""
    obj = APIError.__new__(APIError)
    obj.__dict__.update(state)
    Exception.__init__(obj, f'{obj.code} {obj.status}. {obj.details}')
    return obj

  def _get_status(self, response_json: Any) -> Any:
    try:
      status = response_json.get(
          'status', response_json.get('error', {}).get('status', None)
      )
      return status
    except AttributeError:
      # If response_json is not a dict, return close code to handle the case
      # when encountering a websocket error.
      return None

  def _get_message(self, response_json: Any) -> Any:
    try:
      message = response_json.get(
          'message', response_json.get('error', {}).get('message', None)
      )
      return message
    except AttributeError:
      # If response_json is not a dict, return it as None.
      # This is to handle the case when encountering a websocket error.
      return None

  def _get_code(self, response_json: Any) -> Any:
    return response_json.get(
        'code', response_json.get('error', {}).get('code', None)
    )

  def _to_replay_record(self) -> _common.StringDict:
    """Returns a dictionary representation of the error for replay recording.

    details is not included since it may expose internal information in the
    replay file.
    """
    return {
        'error': {
            'code': self.code,
            'message': self.message,
            'status': self.status,
        }
    }

  @classmethod
  def raise_for_response(
      cls, response: Union['ReplayResponse', httpx.Response, requests.Response]
  ) -> None:
    """Raises an error with detailed error message if the response has an error status."""
    if response.status_code == 200:
      return

    if isinstance(response, httpx.Response):
      try:
        response.read()
        response_json = response.json()
      except json.decoder.JSONDecodeError:
        message = response.text
        response_json = {
            'message': message,
            'status': response.reason_phrase,
        }
    elif isinstance(response, requests.Response):
      try:
        # do not do any extra muanipulation on the response.
        # return the raw response json as is.
        response_json = response.json()
      except requests.exceptions.JSONDecodeError:
        response_json = {
            'message': response.text,
            'status': response.reason,
        }
    else:
      response_json = response.body_segments[0].get('error', {})

    cls.raise_error(response.status_code, response_json, response)

  @classmethod
  def raise_error(
      cls,
      status_code: int,
      response_json: Any,
      response: Optional[
          Union[
              'ReplayResponse',
              httpx.Response,
              requests.Response,
          ]
      ],
  ) -> None:
    """Raises an appropriate APIError subclass based on the status code.

    Args:
      status_code: The HTTP status code of the response.
      response_json: The JSON body of the response, or a dict containing error
        details.
      response: The original response object.

    Raises:
      ClientError: If the status code is in the 4xx range.
      ServerError: If the status code is in the 5xx range.
      APIError: For other error status codes.
    """
    if 400 <= status_code < 500:
      raise ClientError(status_code, response_json, response)
    elif 500 <= status_code < 600:
      raise ServerError(status_code, response_json, response)
    else:
      raise cls(status_code, response_json, response)

  @classmethod
  async def raise_for_async_response(
      cls,
      response: Union[
          'ReplayResponse',
          httpx.Response,
          'aiohttp.ClientResponse',
          'AsyncAuthorizedSessionResponse',
      ],
  ) -> None:
    """Raises an error with detailed error message if the response has an error status."""
    if isinstance(response, httpx.Response):
      if response.status_code == 200:
        return
      try:
        await response.aread()
        response_json = response.json()
      except json.decoder.JSONDecodeError:
        message = response.text
        response_json = {
            'message': message,
            'status': response.reason_phrase,
        }
      status_code = response.status_code
    elif hasattr(response, 'body_segments') and hasattr(
        response, 'status_code'
    ):
      if response.status_code == 200:
        return
      response_json = response.body_segments[0].get('error', {})
      status_code = response.status_code
    else:
      try:
        import aiohttp  # pylint: disable=g-import-not-at-top

        # Use a local variable to help Mypy handle the unwrapped response
        unwrapped_response: Any = response
        if hasattr(unwrapped_response, '_response'):
          unwrapped_response = unwrapped_response._response

        if isinstance(unwrapped_response, aiohttp.ClientResponse):
          if unwrapped_response.status == 200:
            return
          try:
            response_json = await unwrapped_response.json()
          except aiohttp.client_exceptions.ContentTypeError:
            message = await unwrapped_response.text()
            response_json = {
                'message': message,
                'status': unwrapped_response.reason,
            }
          status_code = unwrapped_response.status
        else:
          raise ValueError(f'Unsupported response type: {type(response)}')
      except ImportError:
        raise ValueError(f'Unsupported response type: {type(response)}')

    await cls.raise_error_async(status_code, response_json, response)

  @classmethod
  async def raise_error_async(
      cls, status_code: int, response_json: Any, response: Optional[
          Union['ReplayResponse', httpx.Response, 'aiohttp.ClientResponse']
      ]
  ) -> None:
    """Raises an appropriate APIError subclass based on the status code.

    Args:
      status_code: The HTTP status code of the response.
      response_json: The JSON body of the response, or a dict containing error
        details.
      response: The original response object.

    Raises:
      ClientError: If the status code is in the 4xx range.
      ServerError: If the status code is in the 5xx range.
      APIError: For other error status codes.
    """
    if 400 <= status_code < 500:
      raise ClientError(status_code, response_json, response)
    elif 500 <= status_code < 600:
      raise ServerError(status_code, response_json, response)
    else:
      raise cls(status_code, response_json, response)


class ClientError(APIError):
  """Client error raised by the GenAI API."""
  pass


class ServerError(APIError):
  """Server error raised by the GenAI API."""
  pass


class UnknownFunctionCallArgumentError(ValueError):
  """Raised when the function call argument cannot be converted to the parameter annotation."""
  pass


class UnsupportedFunctionError(ValueError):
  """Raised when the function is not supported."""
  pass


class FunctionInvocationError(ValueError):
  """Raised when the function cannot be invoked with the given arguments."""
  pass


class UnknownApiResponseError(ValueError):
  """Raised when the response from the API cannot be parsed as JSON."""
  pass

ExperimentalWarning = _common.ExperimentalWarning



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# !Please DO NOT combine the test in this file with other tests. This file is
# used to test the future annotation, which is the first line of import.
from __future__ import annotations

import copy
import pydantic

from ... import types


class ComplexType(pydantic.BaseModel):
  param_x: str
  param_y: int


class FakeClient:

  def __init__(self, vertexai=False) -> None:
    self.vertexai = vertexai


mldev_client = FakeClient()
vertex_client = FakeClient(vertexai=True)


def test_future_annotation_simple_type():
  def func_under_test(param_1: str, param_2: int) -> str:
    return '123'

  expected_schema_mldev = types.FunctionDeclaration(
      name='func_under_test',
      parameters=types.Schema(
          type='OBJECT',
          properties={
              'param_1': types.Schema(type='STRING'),
              'param_2': types.Schema(type='INTEGER'),
          },
          required=['param_1', 'param_2'],
      ),
  )
  expected_schema_vertex = copy.deepcopy(expected_schema_mldev)
  expected_schema_vertex.response = types.Schema(type='STRING')
  actual_schema_mldev = types.FunctionDeclaration.from_callable(
      client=mldev_client, callable=func_under_test
  )
  actual_schema_vertex = types.FunctionDeclaration.from_callable(
      client=vertex_client, callable=func_under_test
  )
  assert actual_schema_mldev == expected_schema_mldev
  assert actual_schema_vertex == expected_schema_vertex


def test_future_annotation_complex_type():
  def func_under_test(param_1: ComplexType, param_2: int) -> str:
    return '123'

  expected_schema_mldev = types.FunctionDeclaration(
      name='func_under_test',
      parameters=types.Schema(
          type='OBJECT',
          properties={
              'param_1': types.Schema(
                  type='OBJECT',
                  properties={
                      'param_x': types.Schema(type='STRING'),
                      'param_y': types.Schema(type='INTEGER'),
                  },
                  required=['param_x', 'param_y'],
              ),
              'param_2': types.Schema(type='INTEGER'),
          },
          required=['param_1', 'param_2'],
      )
  )
  expected_schema_vertex = copy.deepcopy(expected_schema_mldev)
  expected_schema_vertex.response = types.Schema(type='STRING')

  actual_schema_mldev = types.FunctionDeclaration.from_callable(
      client=mldev_client, callable=func_under_test
  )
  actual_schema_vertex = types.FunctionDeclaration.from_callable(
      client=vertex_client, callable=func_under_test
  )

  assert actual_schema_mldev == expected_schema_mldev
  assert actual_schema_vertex == expected_schema_vertex



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""[Experimental] Auth Tokens API client."""

import json
import logging
from typing import Any, Dict, List, Optional
from urllib.parse import urlencode
from . import _api_module
from . import _common
from . import _tokens_converters as tokens_converters
from . import types

logger = logging.getLogger('google_genai.tokens')


def _get_field_masks(setup: _common.StringDict) -> str:
  """Return field_masks"""
  fields = []
  for k, v in setup.items():
    # 2nd layer, recursively get field masks see TODO(b/418290100)
    if isinstance(v, dict) and v:
      field = [f'{k}.{kk}' for kk in v.keys()]
    else:
      field = [k]  # 1st layer
    fields.extend(field)

  return ','.join(fields)


def _convert_bidi_setup_to_token_setup(
    request_dict: _common.StringDict,
    config: Optional[types.CreateAuthTokenConfigOrDict] = None,
) -> _common.StringDict:
  """Converts bidiGenerateContentSetup."""
  bidi_setup = request_dict.get('bidiGenerateContentSetup')
  if bidi_setup and bidi_setup.get('setup'):
    # Handling mismatch between AuthToken service and
    # BidiGenerateContent service
    request_dict['bidiGenerateContentSetup'] = bidi_setup.get('setup')

    # Convert non null bidiGenerateContentSetup to field_mask
    field_mask = _get_field_masks(request_dict['bidiGenerateContentSetup'])

    if (
        isinstance(config, dict)
        and config.get('lock_additional_fields') is not None
        and not config.get('lock_additional_fields')
    ) or (
        isinstance(config, types.CreateAuthTokenConfig)
        and config.lock_additional_fields is not None
        and not config.lock_additional_fields  # pylint: disable=literal-comparison
    ):
      # Empty list, lock non null fields
      request_dict['fieldMask'] = field_mask
    elif (
        isinstance(config, dict)
        and config.get('lock_additional_fields') is None
    ) or (
        isinstance(config, types.CreateAuthTokenConfig)
        and config.lock_additional_fields is None
    ):
      # None. Global lock. unset fieldMask
      request_dict.pop('fieldMask', None)
    elif request_dict['fieldMask']:
      # Lock non null + additional fields
      additional_fields_list: Optional[List[str]] = request_dict.get(
          'fieldMask'
      )
      generation_config_list = types.GenerationConfig().model_dump().keys()
      if additional_fields_list:
        field_mask_list = []
        for field in additional_fields_list:
          if field in generation_config_list:
            field = f'generationConfig.{field}'
          field_mask_list.append(field)
      else:
        field_mask_list = []
      request_dict['fieldMask'] = (
          field_mask + ',' + ','.join(field_mask_list)
          if field_mask_list
          else field_mask
      )
    else:
      # Lock all fields
      request_dict.pop('fieldMask', None)
  else:
    field_mask = request_dict.get('fieldMask', [])
    field_mask_str = ','.join(field_mask)
    if field_mask:
      request_dict['fieldMask'] = field_mask_str
    else:
      request_dict.pop('fieldMask', None)
  if not request_dict.get('bidiGenerateContentSetup'):
    request_dict.pop('bidiGenerateContentSetup', None)

  return request_dict


class Tokens(_api_module.BaseModule):
  """[Experimental] Auth Tokens API client.

  This class provides methods for creating auth tokens.
  """

  @_common.experimental_warning(
      "The SDK's token creation implementation is experimental, "
      'and may change in future versions.',
  )
  def create(
      self, *, config: Optional[types.CreateAuthTokenConfigOrDict] = None
  ) -> types.AuthToken:
    """[Experimental] Creates an auth token.

    Args:
      config (CreateAuthTokenConfig): Optional configuration for the request.

    The CreateAuthTokenConfig's `live_constrained_parameters` attrubite
    Can be used to lock the parameters of the live session so they
    can't be changed client side. This behavior has two basic modes depending on
    whether `lock_additional_fields` is set:

    If you do not pass `lock_additional_fields` the entire
    `live_constrained_parameters` is locked and can't be changed
    by the token's user.

    If you set `lock_additional_fields`, then the non-null fields of
    `live_constrained_parameters` are locked, and any additional fields
    specified in `lock_additional_fields`.

    Usage:

    .. code-block:: python

      # Case 1: If LiveEphemeralParameters is unset, unlock LiveConnectConfig
      # when using the token in Live API sessions. Each session connection can
      # use a different configuration.

      config = types.CreateAuthTokenConfig(
          uses=10,
          expire_time='2025-05-01T00:00:00Z',
      )
      auth_token = client.tokens.create(config=config)

    .. code-block:: python

      # Case 2: If LiveEphemeralParameters is set, lock all fields in
      # LiveConnectConfig when using the token in Live API sessions. For
      # example, changing `output_audio_transcription` in the Live API
      # connection will be ignored by the API.

      auth_token = client.tokens.create(
          config=types.CreateAuthTokenConfig(
              uses=10,
              live_constrained_parameters=types.LiveEphemeralParameters(
                  model='gemini-live-2.5-flash-preview',
                  config=types.LiveConnectConfig(
                      system_instruction='You are an LLM called Gemini.'
                  ),
              ),
          )
      )

    .. code-block:: python

      # Case 3: If LiveEphemeralParameters is set and lockAdditionalFields is
      # empty, lock LiveConnectConfig with set fields (e.g.
      # system_instruction in this example) when using the token in Live API
      # sessions.
      auth_token = client.tokens.create(
          config=types.CreateAuthTokenConfig(
              uses=10,
              live_constrained_parameters=types.LiveEphemeralParameters(
                  config=types.LiveConnectConfig(
                      system_instruction='You are an LLM called Gemini.'
                  ),
              ),
              lock_additional_fields=[],
          )
      )

    .. code-block:: python

      # Case 4: If LiveEphemeralParameters is set and lockAdditionalFields is
      # set, lock LiveConnectConfig with set and additional fields (e.g.
      # system_instruction, temperature in this example) when using the token
      # in Live API sessions.
      auth_token = client.tokens.create(
          config=types.CreateAuthTokenConfig(
              uses=10,
              live_constrained_parameters=types.LiveEphemeralParameters(
                  model='gemini-live-2.5-flash-preview',
                  config=types.LiveConnectConfig(
                      system_instruction='You are an LLM called Gemini.'
                  ),
              ),
              lock_additional_fields=['temperature'],
          )
      )
    """

    parameter_model = types.CreateAuthTokenParameters(
        config=config,
    )
    request_url_dict: Optional[dict[str, str]]
    if self._api_client.vertexai:
      raise ValueError(
          'This method is only supported in the Gemini Developer client.'
      )
    else:
      request_dict = tokens_converters._CreateAuthTokenParameters_to_mldev(
          self._api_client,
          parameter_model
      )
      request_url_dict = request_dict.get('_url')
      if request_url_dict:
        path = 'auth_tokens'.format_map(request_url_dict)
      else:
        path = 'auth_tokens'

    query_params = request_dict.get('_query')
    if query_params:
      path = f'{path}?{urlencode(query_params)}'
    # TODO: remove the hack that pops config.

    request_dict.pop('config', None)

    # Token creation request data need to replace 'setup' with
    # 'bidiGenerateContentSetup'
    if request_dict:
      request_dict = _convert_bidi_setup_to_token_setup(request_dict, config)

    http_options: Optional[types.HttpOptions] = None
    if (
        parameter_model is not None
        and parameter_model.config is not None
        and parameter_model.config.http_options is not None
    ):
      http_options = parameter_model.config.http_options

    request_dict = _common.convert_to_dict(request_dict)
    request_dict = _common.encode_unserializable_types(request_dict)

    response = self._api_client.request(
        'post', path, request_dict, http_options
    )
    response_dict = {} if not response.body else json.loads(response.body)

    return_value = types.AuthToken._from_response(
        response=response_dict, kwargs=parameter_model.model_dump()
    )
    self._api_client._verify_response(return_value)
    return return_value


class AsyncTokens(_api_module.BaseModule):
  """[Experimental] Async Auth Tokens API client.

  This class provides asynchronous methods for creating auth tokens.
  """

  @_common.experimental_warning(
      "The SDK's token creation implementation is experimental, "
      'and may change in future versions.',
  )
  async def create(
      self, *, config: Optional[types.CreateAuthTokenConfigOrDict] = None
  ) -> types.AuthToken:
    """Creates an auth token asynchronously. Support in v1alpha only.

    Args:
      config (CreateAuthTokenConfig): Optional configuration for the request.

    Usage:

    .. code-block:: python

      client = genai.Client(
          api_key=API_KEY,
          http_options=types.HttpOptions(api_version='v1alpha'),
      )

      auth_token = await client.aio.tokens.create(
          config=types.CreateAuthTokenConfig(
              uses=10,
              live_constrained_parameters=types.LiveEphemeralParameters(
                  model='gemini-live-2.5-flash-preview',
                  config=types.LiveConnectConfig(
                      system_instruction='You are an LLM called Gemini.'
                  ),
              ),
          )
      )
    """

    parameter_model = types.CreateAuthTokenParameters(
        config=config,
    )

    request_url_dict: Optional[dict[str, str]]
    if self._api_client.vertexai:
      raise ValueError(
          'This method is only supported in the Gemini Developer client.'
      )
    else:
      request_dict = tokens_converters._CreateAuthTokenParameters_to_mldev(
          self._api_client,
          parameter_model
      )
      request_url_dict = request_dict.get('_url')
      if request_url_dict:
        path = 'auth_tokens'.format_map(request_url_dict)
      else:
        path = 'auth_tokens'

    query_params = request_dict.get('_query')
    if query_params:
      path = f'{path}?{urlencode(query_params)}'
    # TODO: remove the hack that pops config.
    request_dict.pop('config', None)

    # Token creation request data need to replace 'setup' with
    # 'bidiGenerateContentSetup'
    request_dict = _convert_bidi_setup_to_token_setup(request_dict, config)

    http_options: Optional[types.HttpOptions] = None
    if (
        parameter_model is not None
        and parameter_model.config is not None
        and parameter_model.config.http_options is not None
    ):
      http_options = parameter_model.config.http_options

    request_dict = _common.convert_to_dict(request_dict)
    request_dict = _common.encode_unserializable_types(request_dict)

    response = await self._api_client.async_request(
        'post',
        path,
        request_dict,
        http_options=http_options,
    )
    response_dict = {} if not response.body else json.loads(response.body)

    return_value = types.AuthToken._from_response(
        response=response_dict, kwargs=parameter_model.model_dump()
    )
    self._api_client._verify_response(return_value)
    return return_value



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


import pytest
from ... import types


@pytest.fixture
def generate_content_response():
  return types.GenerateContentResponse()


def test_candidate_empty_text():
  response = types.GenerateContentResponse()
  assert response.text is None


def test_first_candidate_empty_content_text():
  response = types.GenerateContentResponse(candidates=[])

  # MUST assert None not implicit boolean False
  assert response.text is None


def test_first_candidate_empty_parts_text():
  response = types.GenerateContentResponse(candidates=[types.Candidate()])

  # MUST assert None not implicit boolean False
  assert response.text is None


def test_content_empty_parts_text():
  response = types.GenerateContentResponse(
      candidates=[types.Candidate(content=types.Content())]
  )

  # MUST assert None not implicit boolean False
  assert response.text is None


def test_two_candidates_text(caplog, generate_content_response):
  from ... import types as types_module
  types_module._response_text_warning_logged = False

  generate_content_response.candidates = [
      types.Candidate(
          content=types.Content(
              parts=[
                  types.Part(text='Hello1'),
                  types.Part(text='World1'),
              ]
          )
      ),
      types.Candidate(
          content=types.Content(
              parts=[
                  types.Part(text='Hello2'),
                  types.Part(text='World2'),
              ]
          )
      ),
  ]

  assert generate_content_response.text == 'Hello1World1'
  assert any(
      record.levelname == 'WARNING'
      and 'there are 2 candidates' in record.message
      for record in caplog.records
  )
  generate_content_response.candidates = None


def test_thought_signature_no_warning(caplog, generate_content_response):
  generate_content_response.candidates = [
      types.Candidate(
          content=types.Content(
              parts=[
                  types.Part(text='Hello'),
                  types.Part(thought_signature=b'thought'),
              ]
          )
      ),
  ]

  assert generate_content_response.text == 'Hello'
  assert not any(
      record.levelname == 'WARNING'
      and 'there are non-text parts in the response' in record.message
      for record in caplog.records
  )
  generate_content_response.candidates = None


def test_thought_signature_no_warning_in_text(
    caplog, generate_content_response
):
  generate_content_response.candidates = [
      types.Candidate(
          content=types.Content(
              parts=[
                  types.Part(text='Hello', thought_signature=b'thought'),
              ]
          )
      ),
  ]

  assert generate_content_response.text == 'Hello'
  assert not any(
      record.levelname == 'WARNING'
      and 'there are non-text parts in the response' in record.message
      for record in caplog.records
  )
  generate_content_response.candidates = None


def test_1_candidate_text():
  response = types.GenerateContentResponse(
      candidates=[
          types.Candidate(
              content=types.Content(
                  parts=[
                      types.Part(text='Hello1'),
                      types.Part(text='World1'),
                  ]
              )
          )
      ]
  )

  assert response.text == 'Hello1World1'


def test_all_empty_text_in_parts():
  response = types.GenerateContentResponse(
      candidates=[
          types.Candidate(
              content=types.Content(
                  parts=[
                      types.Part(text=''),
                      types.Part(text=''),
                  ]
              )
          ),
          types.Candidate(
              content=types.Content(
                  parts=[
                      types.Part(text='Hello2'),
                      types.Part(text='World2'),
                  ]
              )
          ),
      ]
  )

  # MUST assert empty string, not implicit boolean False
  assert response.text == ''


def test_one_empty_text_in_parts():
  response = types.GenerateContentResponse(
      candidates=[
          types.Candidate(
              content=types.Content(
                  parts=[
                      types.Part(text=''),
                      types.Part(text='World1'),
                  ]
              )
          ),
          types.Candidate(
              content=types.Content(
                  parts=[
                      types.Part(text='Hello2'),
                      types.Part(text='World2'),
                  ]
              )
          ),
      ]
  )

  assert response.text == 'World1'


def test_all_none_text():
  response = types.GenerateContentResponse(
      candidates=[
          types.Candidate(
              content=types.Content(
                  parts=[
                      types.Part(),
                      types.Part(),
                  ]
              )
          ),
          types.Candidate(
              content=types.Content(
                  parts=[
                      types.Part(text='Hello2'),
                      types.Part(text='World2'),
                  ]
              )
          ),
      ]
  )

  # MUST assert None not implicit boolean False
  assert response.text is None


def test_none_empty_text():
  response = types.GenerateContentResponse(
      candidates=[
          types.Candidate(
              content=types.Content(
                  parts=[
                      types.Part(),
                      types.Part(text=''),
                  ]
              )
          ),
          types.Candidate(
              content=types.Content(
                  parts=[
                      types.Part(text='Hello2'),
                      types.Part(text='World2'),
                  ]
              )
          ),
      ]
  )

  # MUST assert empty string, not implicit boolean False
  assert response.text == ''


def test_non_text_part_text(caplog, generate_content_response):
  from ... import types as types_module
  types_module._response_text_non_text_warning_logged = False

  generate_content_response.candidates = [
      types.Candidate(
          content=types.Content(
              parts=[
                  types.Part(function_call=types.FunctionCall()),
              ]
          )
      ),
  ]
  assert generate_content_response.text is None
  assert any(
      record.levelname == 'WARNING'
      and 'there are non-text parts in the response' in record.message
      for record in caplog.records
  )
  generate_content_response.candidates = None


def test_non_text_part_and_text_part_text(caplog, generate_content_response):
  generate_content_response.candidates = [
      types.Candidate(
          content=types.Content(
              parts=[
                  types.Part(function_call=types.FunctionCall()),
                  types.Part(text='World1'),
              ]
          )
      ),
  ]

  assert generate_content_response.text == 'World1'
  assert not any(
      record.levelname == 'WARNING'
      and 'there are no text parts in the response' in record.message
      for record in caplog.records
  )
  generate_content_response.candidates = None


def test_candidates_none_function_calls():
  response = types.GenerateContentResponse()
  assert response.function_calls is None


def test_candidates_empty_function_calls():
  response = types.GenerateContentResponse(candidates=[])
  assert response.function_calls is None


def test_content_none_function_calls():
  response = types.GenerateContentResponse(candidates=[types.Candidate()])
  assert response.function_calls is None


def test_parts_none_function_calls():
  response = types.GenerateContentResponse(
      candidates=[types.Candidate(content=types.Content())]
  )
  assert response.function_calls is None


def test_parts_empty_function_calls():
  response = types.GenerateContentResponse(
      candidates=[
          types.Candidate(content=types.Content(parts=[])),
      ]
  )
  assert response.function_calls is None


def test_multiple_candidates_function_calls(caplog, generate_content_response):
  generate_content_response.candidates = [
      types.Candidate(
          content=types.Content(
              parts=[
                  types.Part(
                      function_call=types.FunctionCall.model_validate({
                          'args': {'key1': 'value1'},
                          'name': 'funcCall1',
                      })
                  )
              ]
          )
      ),
      types.Candidate(
          content=types.Content(
              parts=[
                  types.Part(
                      function_call=types.FunctionCall.model_validate({
                          'args': {'key2': 'value2'},
                          'name': 'funcCall2',
                      })
                  )
              ]
          )
      ),
  ]
  assert generate_content_response.function_calls == [
      types.FunctionCall(name='funcCall1', args={'key1': 'value1'})
  ]
  assert any(
      record.levelname == 'WARNING'
      and 'there are multiple candidates in the response' in record.message
      for record in caplog.records
  )
  generate_content_response.candidates = None


def test_multiple_function_calls():
  response = types.GenerateContentResponse(
      candidates=[
          types.Candidate(
              content=types.Content(
                  parts=[
                      types.Part(
                          function_call=types.FunctionCall.model_validate({
                              'args': {'key1': 'value1'},
                              'name': 'funcCall1',
                          })
                      ),
                      types.Part(
                          function_call=types.FunctionCall.model_validate({
                              'args': {'key2': 'value2'},
                              'name': 'funcCall2',
                          })
                      ),
                  ]
              )
          ),
      ]
  )
  assert response.function_calls == [
      types.FunctionCall(name='funcCall1', args={'key1': 'value1'}),
      types.FunctionCall(name='funcCall2', args={'key2': 'value2'}),
  ]


def test_no_function_calls():
  response = types.GenerateContentResponse(
      candidates=[
          types.Candidate(
              content=types.Content(
                  parts=[
                      types.Part(text='Hello1'),
                      types.Part(text='World1'),
                  ]
              )
          ),
      ]
  )
  assert response.function_calls is None


def test_executable_code_empty_candidates():
  response = types.GenerateContentResponse()

  assert response.executable_code is None


def test_executable_code_empty_content():
  response = types.GenerateContentResponse(candidates=[])

  assert response.executable_code is None


def test_executable_code_empty_parts():
  response = types.GenerateContentResponse(candidates=[types.Candidate(
      content=types.Content()
  )])

  assert response.executable_code is None


def test_executable_code_two_candidates(caplog, generate_content_response):
  generate_content_response.candidates = [
      types.Candidate(
          content=types.Content(
              parts=[
                  types.Part(
                      executable_code=types.ExecutableCode(
                          code='print("hello")', language='PYTHON'
                      )
                  )
              ]
          )
      ),
      types.Candidate(
          content=types.Content(
              parts=[
                  types.Part(
                      executable_code=types.ExecutableCode(
                          code='print("world")', language='PYTHON'
                      )
                  )
              ]
          )
      ),
  ]

  assert generate_content_response.executable_code == 'print("hello")'
  assert any(
      record.levelname == 'WARNING'
      and 'there are multiple candidates in the response' in record.message
      for record in caplog.records
  )
  generate_content_response.candidates = None

def test_executable_code_one_candidate():
  response = types.GenerateContentResponse(
      candidates=[
          types.Candidate(
              content=types.Content(
                  parts=[
                      types.Part(
                          executable_code=types.ExecutableCode(
                              code='print("hello")', language='PYTHON'
                          )
                      )
                  ]
              )
          )
      ]
  )

  assert response.executable_code == 'print("hello")'


def test_code_execution_result_empty_candidates():
  response = types.GenerateContentResponse()

  assert response.code_execution_result is None, response.code_execution_result


def test_code_execution_result_empty_content():
  response = types.GenerateContentResponse(candidates=[])

  assert response.code_execution_result is None


def test_code_execution_result_empty_parts():
  response = types.GenerateContentResponse(
      candidates=[types.Candidate(content=types.Content())]
  )

  assert response.code_execution_result is None


def test_code_execution_result_two_candidates(caplog, generate_content_response):
  generate_content_response.candidates = [
      types.Candidate(
          content=types.Content(
              parts=[
                  types.Part(
                      code_execution_result=types.CodeExecutionResult(
                          outcome='OUTCOME_OK', output='"hello"'
                      )
                  )
              ]
          )
      ),
      types.Candidate(
          content=types.Content(
              parts=[
                  types.Part(
                      code_execution_result=types.CodeExecutionResult(
                          outcome='OUTCOME_ERROR', output='"world"'
                      )
                  )
              ]
          )
      ),
  ]

  assert generate_content_response.code_execution_result == '"hello"'
  assert any (
      record.levelname == 'WARNING'
      and 'there are multiple candidates in the response' in record.message
      for record in caplog.records
  )
  generate_content_response.candidates = None


def test_code_execution_result_one_candidate():
  response = types.GenerateContentResponse(
      candidates=[
          types.Candidate(
              content=types.Content(
                  parts=[
                      types.Part(
                          code_execution_result=types.CodeExecutionResult(
                              outcome='OUTCOME_OK', output='"hello"'
                          )
                      )
                  ]
              )
          )
      ]
  )

  assert response.code_execution_result == '"hello"'


def test_from_file_media_resolution_str():
  file_uri = types.Part.from_uri(
      file_uri='gs://test',
      mime_type='image/png',
      media_resolution='MEDIA_RESOLUTION_LOW',
  )
  assert file_uri.file_data.file_uri == 'gs://test'
  assert file_uri.file_data.mime_type == 'image/png'
  assert file_uri.media_resolution.level == types.PartMediaResolutionLevel.MEDIA_RESOLUTION_LOW


def test_from_file_media_resolution_enum():
  file_uri = types.Part.from_uri(
      file_uri='gs://test',
      mime_type='image/png',
      media_resolution=types.PartMediaResolutionLevel.MEDIA_RESOLUTION_LOW,
  )
  assert file_uri.file_data.file_uri == 'gs://test'
  assert file_uri.file_data.mime_type == 'image/png'
  assert file_uri.media_resolution.level == types.PartMediaResolutionLevel.MEDIA_RESOLUTION_LOW


def test_from_file_media_resolution_object():
  file_uri = types.Part.from_uri(
      file_uri='gs://test',
      mime_type='image/png',
      media_resolution=types.PartMediaResolution(level='MEDIA_RESOLUTION_LOW'),
  )
  assert file_uri.file_data.file_uri == 'gs://test'
  assert file_uri.file_data.mime_type == 'image/png'
  assert file_uri.media_resolution.level == types.PartMediaResolutionLevel.MEDIA_RESOLUTION_LOW


def test_from_bytes_media_resolution_str():
  file_uri = types.Part.from_bytes(
      data=b'1234',
      mime_type='image/png',
      media_resolution='MEDIA_RESOLUTION_LOW',
  )
  assert file_uri.inline_data.data == b'1234'
  assert file_uri.inline_data.mime_type == 'image/png'
  assert file_uri.media_resolution.level == types.PartMediaResolutionLevel.MEDIA_RESOLUTION_LOW


def test_from_bytes_media_resolution_enum():
  file_uri = types.Part.from_bytes(
      data=b'1234',
      mime_type='image/png',
      media_resolution=types.PartMediaResolutionLevel.MEDIA_RESOLUTION_LOW,
  )
  assert file_uri.inline_data.data == b'1234'
  assert file_uri.inline_data.mime_type == 'image/png'
  assert file_uri.media_resolution.level == types.PartMediaResolutionLevel.MEDIA_RESOLUTION_LOW


def test_from_bytes_media_resolution_object():
  file_uri = types.Part.from_bytes(
      data=b'1234',
      mime_type='image/png',
      media_resolution=types.PartMediaResolution(level='MEDIA_RESOLUTION_LOW'),
  )
  assert file_uri.inline_data.data == b'1234'
  assert file_uri.inline_data.mime_type == 'image/png'
  assert file_uri.media_resolution.level == types.PartMediaResolutionLevel.MEDIA_RESOLUTION_LOW



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import _asyncio
import pytest
from ... import _extra_utils
from ... import types
from ..._adapters import McpToGenAiToolAdapter

try:
  from mcp import types as mcp_types
  from mcp import ClientSession as McpClientSession
except ImportError as e:
  import sys

  if sys.version_info < (3, 10):
    raise ImportError(
        'MCP Tool requires Python 3.10 or above. Please upgrade your Python'
        ' version.'
    ) from e
  else:
    raise e


@pytest.mark.asyncio
async def test_parse_empty_config_dict():
  """Test conversion of empty GenerateContentConfigDict to parsed config."""
  config = {}
  parsed_config, mcp_to_genai_tool_adapters = (
      await _extra_utils.parse_config_for_mcp_sessions(config)
  )
  assert parsed_config == None
  assert not mcp_to_genai_tool_adapters


@pytest.mark.asyncio
async def test_parse_empty_config_object():
  """Test conversion of empty GenerateContentConfig to parsed config."""
  config = types.GenerateContentConfig()
  parsed_config, mcp_to_genai_tool_adapters = (
      await _extra_utils.parse_config_for_mcp_sessions(config)
  )
  assert config is not parsed_config  # config is not modified
  assert not mcp_to_genai_tool_adapters


@pytest.mark.asyncio
async def test_parse_config_object_with_tools():
  """Test conversion of GenerateContentConfig with tools to parsed config."""

  class MockMcpClientSession(McpClientSession):

    def __init__(self):
      self._read_stream = None
      self._write_stream = None

    async def list_tools(self):
      return mcp_types.ListToolsResult(
          tools=[
              mcp_types.Tool(
                  name='get_weather',
                  description='Get the weather in a city.',
                  inputSchema={
                      'type': 'object',
                      'properties': {'location': {'type': 'string'}},
                  },
              ),
              mcp_types.Tool(
                  name='get_weather_2',
                  description='Different tool to get the weather.',
                  inputSchema={
                      'type': 'object',
                      'properties': {'location': {'type': 'string'}},
                  },
              ),
          ]
      )

  mock_session_instance = MockMcpClientSession()
  config = types.GenerateContentConfig(tools=[mock_session_instance])
  parsed_config, mcp_to_genai_tool_adapters = (
      await _extra_utils.parse_config_for_mcp_sessions(config)
  )
  assert len(config.tools) == 1
  assert config.tools[0] is mock_session_instance
  assert config is not parsed_config  # config is not modified
  assert len(mcp_to_genai_tool_adapters) == 2
  assert mcp_to_genai_tool_adapters.keys() == {
      'get_weather',
      'get_weather_2',
  }
  assert isinstance(
      mcp_to_genai_tool_adapters['get_weather'], McpToGenAiToolAdapter
  )
  assert isinstance(
      mcp_to_genai_tool_adapters['get_weather_2'], McpToGenAiToolAdapter
  )


@pytest.mark.asyncio
async def test_parse_config_object_with_tools_complex_type():
  """Test conversion of GenerateContentConfig with tools to parsed config."""

  class MockMcpClientSession(McpClientSession):

    def __init__(self):
      self._read_stream = None
      self._write_stream = None
      self._future = _asyncio.Future()  # This object cannot be pickled.

    async def list_tools(self):
      return mcp_types.ListToolsResult(
          tools=[
              mcp_types.Tool(
                  name='get_weather',
                  description='Get the weather in a city.',
                  inputSchema={
                      'type': 'object',
                      'properties': {'location': {'type': 'string'}},
                  },
              ),
              mcp_types.Tool(
                  name='get_weather_2',
                  description='Different tool to get the weather.',
                  inputSchema={
                      'type': 'object',
                      'properties': {'location': {'type': 'string'}},
                  },
              ),
          ]
      )

  mock_session_instance = MockMcpClientSession()
  config = types.GenerateContentConfig(tools=[mock_session_instance])
  parsed_config, mcp_to_genai_tool_adapters = (
      await _extra_utils.parse_config_for_mcp_sessions(config)
  )
  assert len(config.tools) == 1
  assert config.tools[0] is mock_session_instance
  assert config is not parsed_config  # config is not modified
  assert len(mcp_to_genai_tool_adapters) == 2
  assert mcp_to_genai_tool_adapters.keys() == {
      'get_weather',
      'get_weather_2',
  }
  assert isinstance(
      mcp_to_genai_tool_adapters['get_weather'], McpToGenAiToolAdapter
  )
  assert isinstance(
      mcp_to_genai_tool_adapters['get_weather_2'], McpToGenAiToolAdapter
  )


@pytest.mark.asyncio
async def test_parse_config_object_with_non_mcp_tools():
  """Test conversion of GenerateContentConfig with regular tools to parsed config."""

  config = types.GenerateContentConfig(
      tools=[
          types.Tool(
              function_declarations=[
                  {'name': 'tool-1', 'description': 'tool-1-description'}
              ]
          ),
          types.Tool(
              function_declarations=[
                  {'name': 'tool-2', 'description': 'tool-2-description'}
              ]
          ),
      ]
  )
  parsed_config, mcp_to_genai_tool_adapters = (
      await _extra_utils.parse_config_for_mcp_sessions(config)
  )
  assert len(config.tools) == 2
  assert config is not parsed_config  # config is not modified
  assert mcp_to_genai_tool_adapters == {}
  assert parsed_config.tools == [
      types.Tool(
          function_declarations=[
              {'name': 'tool-1', 'description': 'tool-1-description'}
          ]
      ),
      types.Tool(
          function_declarations=[
              {'name': 'tool-2', 'description': 'tool-2-description'}
          ]
      ),
  ]



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


"""Tests for file_search_stores.create()."""

import pytest

from ... import types
from .. import pytest_helper

# All tests will be run for both Vertex and MLDev.
test_table: list[pytest_helper.TestTableItem] = [
    pytest_helper.TestTableItem(
        name="test_display_name",
        parameters=types._CreateFileSearchStoreParameters(
            config=types.CreateFileSearchStoreConfig(
                display_name="My File Search Store"
            )
        ),
        exception_if_vertex="only supported in the Gemini Developer client",
    ),
    pytest_helper.TestTableItem(
        name="test_basic",
        parameters=types._CreateFileSearchStoreParameters(),
        exception_if_vertex="only supported in the Gemini Developer client",
    ),
]

pytestmark = [
    pytest.mark.usefixtures("mock_timestamped_unique_name"),
    pytest_helper.setup(
        file=__file__,
        globals_for_file=globals(),
        test_method="file_search_stores.create",
        test_table=test_table,
    ),
]


@pytest.mark.asyncio
async def test_async_display_name(client):
  with pytest_helper.exception_if_vertex(client, ValueError):
    file_search_store = await client.aio.file_search_stores.create(
        config=types.CreateFileSearchStoreConfig(
            display_name="My File Search Store"
        )
    )


@pytest.mark.asyncio
async def test_async_basic(client):
  with pytest_helper.exception_if_vertex(client, ValueError):
    file_search_store = await client.aio.file_search_stores.create()



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import typing

from ._mcp_utils import mcp_to_gemini_tools
from .types import FunctionCall, Tool

if typing.TYPE_CHECKING:
  from mcp import types as mcp_types
  from mcp import ClientSession


class McpToGenAiToolAdapter:
  """Adapter for working with MCP tools in a GenAI client."""

  def __init__(
      self,
      session: "mcp.ClientSession",  # type: ignore # noqa: F821
      list_tools_result: "mcp_types.ListToolsResult",  # type: ignore
  ) -> None:
    self._mcp_session = session
    self._list_tools_result = list_tools_result

  async def call_tool(
      self, function_call: FunctionCall
  ) -> "mcp_types.CallToolResult":  # type: ignore
    """Calls a function on the MCP server."""
    name = function_call.name if function_call.name else ""
    arguments = dict(function_call.args) if function_call.args else {}

    return typing.cast(
        "mcp_types.CallToolResult",
        await self._mcp_session.call_tool(
            name=name,
            arguments=arguments,
        ),
    )

  @property
  def tools(self) -> list[Tool]:
    """Returns a list of Google GenAI tools."""
    return mcp_to_gemini_tools(self._list_tools_result.tools)



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, Annotated, TypeAlias

from .tool import Tool
from .turn import Turn
from .model import Model
from .usage import Usage
from .._utils import PropertyInfo
from .content import Content
from .._models import BaseModel
from .text_content import TextContent
from .audio_content import AudioContent
from .image_content import ImageContent
from .video_content import VideoContent
from .thought_content import ThoughtContent
from .document_content import DocumentContent
from .dynamic_agent_config import DynamicAgentConfig
from .function_call_content import FunctionCallContent
from .function_result_content import FunctionResultContent
from .file_search_call_content import FileSearchCallContent
from .google_maps_call_content import GoogleMapsCallContent
from .url_context_call_content import URLContextCallContent
from .deep_research_agent_config import DeepResearchAgentConfig
from .file_search_result_content import FileSearchResultContent
from .google_maps_result_content import GoogleMapsResultContent
from .google_search_call_content import GoogleSearchCallContent
from .url_context_result_content import URLContextResultContent
from .code_execution_call_content import CodeExecutionCallContent
from .google_search_result_content import GoogleSearchResultContent
from .mcp_server_tool_call_content import MCPServerToolCallContent
from .code_execution_result_content import CodeExecutionResultContent
from .mcp_server_tool_result_content import MCPServerToolResultContent

__all__ = ["Interaction", "AgentConfig", "Input"]

AgentConfig: TypeAlias = Annotated[
    Union[DynamicAgentConfig, DeepResearchAgentConfig], PropertyInfo(discriminator="type")
]

Input: TypeAlias = Union[
    List[Content],
    str,
    List[Turn],
    TextContent,
    ImageContent,
    AudioContent,
    DocumentContent,
    VideoContent,
    ThoughtContent,
    FunctionCallContent,
    CodeExecutionCallContent,
    URLContextCallContent,
    MCPServerToolCallContent,
    GoogleSearchCallContent,
    FileSearchCallContent,
    GoogleMapsCallContent,
    FunctionResultContent,
    CodeExecutionResultContent,
    URLContextResultContent,
    GoogleSearchResultContent,
    MCPServerToolResultContent,
    FileSearchResultContent,
    GoogleMapsResultContent,
]


class Interaction(BaseModel):
    """The Interaction resource."""

    id: str
    """Required. Output only. A unique identifier for the interaction completion."""

    created: datetime
    """Required.

    Output only. The time at which the response was created in ISO 8601 format
    (YYYY-MM-DDThh:mm:ssZ).
    """

    status: Literal["in_progress", "requires_action", "completed", "failed", "cancelled", "incomplete"]
    """Required. Output only. The status of the interaction."""

    updated: datetime
    """Required.

    Output only. The time at which the response was last updated in ISO 8601 format
    (YYYY-MM-DDThh:mm:ssZ).
    """

    agent: Union[str, Literal["deep-research-pro-preview-12-2025"], None] = None
    """The name of the `Agent` used for generating the interaction."""

    agent_config: Optional[AgentConfig] = None
    """Configuration parameters for the agent interaction."""

    input: Optional[Input] = None
    """The input for the interaction."""

    model: Optional[Model] = None
    """The name of the `Model` used for generating the interaction."""

    outputs: Optional[List[Content]] = None
    """Output only. Responses from the model."""

    previous_interaction_id: Optional[str] = None
    """The ID of the previous interaction, if any."""

    response_format: Optional[object] = None
    """
    Enforces that the generated response is a JSON object that complies with
    the JSON schema specified in this field.
    """

    response_mime_type: Optional[str] = None
    """The mime type of the response. This is required if response_format is set."""

    response_modalities: Optional[List[Literal["text", "image", "audio"]]] = None
    """The requested modalities of the response (TEXT, IMAGE, AUDIO)."""

    role: Optional[str] = None
    """Output only. The role of the interaction."""

    service_tier: Optional[Literal["flex", "standard", "priority"]] = None
    """The service tier for the interaction."""

    system_instruction: Optional[str] = None
    """System instruction for the interaction."""

    tools: Optional[List[Tool]] = None
    """A list of tool declarations the model may call during interaction."""

    usage: Optional[Usage] = None
    """Output only. Statistics on the interaction request's token usage."""



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# Code generated by the Google Gen AI SDK generator DO NOT EDIT.

from functools import partial
import json
import logging
from typing import Any, Optional, Union
from urllib.parse import urlencode

from . import _api_module
from . import _common
from . import types
from ._common import get_value_by_path as getv
from ._common import set_value_by_path as setv
from .pagers import AsyncPager, Pager


logger = logging.getLogger('google_genai.documents')


def _DeleteDocumentConfig_to_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}

  if getv(from_object, ['force']) is not None:
    setv(parent_object, ['_query', 'force'], getv(from_object, ['force']))

  return to_object


def _DeleteDocumentParameters_to_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['name']) is not None:
    setv(to_object, ['_url', 'name'], getv(from_object, ['name']))

  if getv(from_object, ['config']) is not None:
    _DeleteDocumentConfig_to_mldev(getv(from_object, ['config']), to_object)

  return to_object


def _GetDocumentParameters_to_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['name']) is not None:
    setv(to_object, ['_url', 'name'], getv(from_object, ['name']))

  return to_object


def _ListDocumentsConfig_to_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}

  if getv(from_object, ['page_size']) is not None:
    setv(
        parent_object, ['_query', 'pageSize'], getv(from_object, ['page_size'])
    )

  if getv(from_object, ['page_token']) is not None:
    setv(
        parent_object,
        ['_query', 'pageToken'],
        getv(from_object, ['page_token']),
    )

  return to_object


def _ListDocumentsParameters_to_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['parent']) is not None:
    setv(to_object, ['_url', 'parent'], getv(from_object, ['parent']))

  if getv(from_object, ['config']) is not None:
    _ListDocumentsConfig_to_mldev(getv(from_object, ['config']), to_object)

  return to_object


def _ListDocumentsResponse_from_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['sdkHttpResponse']) is not None:
    setv(
        to_object, ['sdk_http_response'], getv(from_object, ['sdkHttpResponse'])
    )

  if getv(from_object, ['nextPageToken']) is not None:
    setv(to_object, ['next_page_token'], getv(from_object, ['nextPageToken']))

  if getv(from_object, ['documents']) is not None:
    setv(
        to_object,
        ['documents'],
        [item for item in getv(from_object, ['documents'])],
    )

  return to_object


class Documents(_api_module.BaseModule):

  def get(
      self, *, name: str, config: Optional[types.GetDocumentConfigOrDict] = None
  ) -> types.Document:
    """Gets metadata about a Document.

    Args:
      name (str): The resource name of the Document.
        Example: ragStores/rag-store-foo/documents/documents-bar
      config (GetDocumentConfig | None): Optional parameters for the request.

    Returns:
      The Document.
    """

    parameter_model = types._GetDocumentParameters(
        name=name,
        config=config,
    )

    request_url_dict: Optional[dict[str, str]]
    if self._api_client.vertexai:
      raise ValueError(
          'This method is only supported in the Gemini Developer client.'
      )
    else:
      request_dict = _GetDocumentParameters_to_mldev(parameter_model)
      request_url_dict = request_dict.get('_url')
      if request_url_dict:
        path = '{name}'.format_map(request_url_dict)
      else:
        path = '{name}'

    query_params = request_dict.get('_query')
    if query_params:
      path = f'{path}?{urlencode(query_params)}'
    # TODO: remove the hack that pops config.
    request_dict.pop('config', None)

    http_options: Optional[types.HttpOptions] = None
    if (
        parameter_model.config is not None
        and parameter_model.config.http_options is not None
    ):
      http_options = parameter_model.config.http_options

    request_dict = _common.convert_to_dict(request_dict)
    request_dict = _common.encode_unserializable_types(request_dict)

    response = self._api_client.request('get', path, request_dict, http_options)

    response_dict = {} if not response.body else json.loads(response.body)

    return_value = types.Document._from_response(
        response=response_dict, kwargs=parameter_model.model_dump()
    )

    self._api_client._verify_response(return_value)
    return return_value

  def delete(
      self,
      *,
      name: str,
      config: Optional[types.DeleteDocumentConfigOrDict] = None,
  ) -> None:
    """Deletes a Document.

    Args:
      name (str): The resource name of the Document.
        Example: ragStores/rag-store-foo/documents/documents-bar
      config (DeleteDocumentConfig | None): Optional parameters for the request.

    Returns:
      None
    """

    parameter_model = types._DeleteDocumentParameters(
        name=name,
        config=config,
    )

    request_url_dict: Optional[dict[str, str]]
    if self._api_client.vertexai:
      raise ValueError(
          'This method is only supported in the Gemini Developer client.'
      )
    else:
      request_dict = _DeleteDocumentParameters_to_mldev(parameter_model)
      request_url_dict = request_dict.get('_url')
      if request_url_dict:
        path = '{name}'.format_map(request_url_dict)
      else:
        path = '{name}'

    query_params = request_dict.get('_query')
    if query_params:
      path = f'{path}?{urlencode(query_params)}'
    # TODO: remove the hack that pops config.
    request_dict.pop('config', None)

    http_options: Optional[types.HttpOptions] = None
    if (
        parameter_model.config is not None
        and parameter_model.config.http_options is not None
    ):
      http_options = parameter_model.config.http_options

    request_dict = _common.convert_to_dict(request_dict)
    request_dict = _common.encode_unserializable_types(request_dict)

    response = self._api_client.request(
        'delete', path, request_dict, http_options
    )

  def _list(
      self,
      *,
      parent: str,
      config: Optional[types.ListDocumentsConfigOrDict] = None,
  ) -> types.ListDocumentsResponse:
    parameter_model = types._ListDocumentsParameters(
        parent=parent,
        config=config,
    )

    request_url_dict: Optional[dict[str, str]]
    if self._api_client.vertexai:
      raise ValueError(
          'This method is only supported in the Gemini Developer client.'
      )
    else:
      request_dict = _ListDocumentsParameters_to_mldev(parameter_model)
      request_url_dict = request_dict.get('_url')
      if request_url_dict:
        path = '{parent}/documents'.format_map(request_url_dict)
      else:
        path = '{parent}/documents'

    query_params = request_dict.get('_query')
    if query_params:
      path = f'{path}?{urlencode(query_params)}'
    # TODO: remove the hack that pops config.
    request_dict.pop('config', None)

    http_options: Optional[types.HttpOptions] = None
    if (
        parameter_model.config is not None
        and parameter_model.config.http_options is not None
    ):
      http_options = parameter_model.config.http_options

    request_dict = _common.convert_to_dict(request_dict)
    request_dict = _common.encode_unserializable_types(request_dict)

    response = self._api_client.request('get', path, request_dict, http_options)

    response_dict = {} if not response.body else json.loads(response.body)

    if not self._api_client.vertexai:
      response_dict = _ListDocumentsResponse_from_mldev(response_dict)

    return_value = types.ListDocumentsResponse._from_response(
        response=response_dict, kwargs=parameter_model.model_dump()
    )

    self._api_client._verify_response(return_value)
    return return_value

  def list(
      self,
      *,
      parent: str,
      config: Optional[types.ListDocumentsConfigOrDict] = None,
  ) -> Pager[types.Document]:
    """Lists documents.

    Args:
      parent (str): The name of the RagStore containing the Documents.
      config (ListDocumentsConfig): Optional configuration for the list request.

    Returns:
      A Pager object that contains one page of documents. When iterating over
      the pager, it automatically fetches the next page if there are more.
    Usage:
    .. code-block:: python
      for document in client.documents.list(parent='rag_store_name'):
        print(f"document: {document.name} - {document.display_name}")
    """

    list_request = partial(self._list, parent=parent)
    return Pager(
        'documents',
        list_request,
        self._list(parent=parent, config=config),
        config,
    )


class AsyncDocuments(_api_module.BaseModule):

  async def get(
      self, *, name: str, config: Optional[types.GetDocumentConfigOrDict] = None
  ) -> types.Document:
    """Gets metadata about a Document.

    Args:
      name (str): The resource name of the Document.
        Example: ragStores/rag-store-foo/documents/documents-bar
      config (GetDocumentConfig | None): Optional parameters for the request.

    Returns:
      The Document.
    """

    parameter_model = types._GetDocumentParameters(
        name=name,
        config=config,
    )

    request_url_dict: Optional[dict[str, str]]
    if self._api_client.vertexai:
      raise ValueError(
          'This method is only supported in the Gemini Developer client.'
      )
    else:
      request_dict = _GetDocumentParameters_to_mldev(parameter_model)
      request_url_dict = request_dict.get('_url')
      if request_url_dict:
        path = '{name}'.format_map(request_url_dict)
      else:
        path = '{name}'

    query_params = request_dict.get('_query')
    if query_params:
      path = f'{path}?{urlencode(query_params)}'
    # TODO: remove the hack that pops config.
    request_dict.pop('config', None)

    http_options: Optional[types.HttpOptions] = None
    if (
        parameter_model.config is not None
        and parameter_model.config.http_options is not None
    ):
      http_options = parameter_model.config.http_options

    request_dict = _common.convert_to_dict(request_dict)
    request_dict = _common.encode_unserializable_types(request_dict)

    response = await self._api_client.async_request(
        'get', path, request_dict, http_options
    )

    response_dict = {} if not response.body else json.loads(response.body)

    return_value = types.Document._from_response(
        response=response_dict, kwargs=parameter_model.model_dump()
    )

    self._api_client._verify_response(return_value)
    return return_value

  async def delete(
      self,
      *,
      name: str,
      config: Optional[types.DeleteDocumentConfigOrDict] = None,
  ) -> None:
    """Deletes a Document.

    Args:
      name (str): The resource name of the Document.
        Example: ragStores/rag-store-foo/documents/documents-bar
      config (DeleteDocumentConfig | None): Optional parameters for the request.

    Returns:
      None
    """

    parameter_model = types._DeleteDocumentParameters(
        name=name,
        config=config,
    )

    request_url_dict: Optional[dict[str, str]]
    if self._api_client.vertexai:
      raise ValueError(
          'This method is only supported in the Gemini Developer client.'
      )
    else:
      request_dict = _DeleteDocumentParameters_to_mldev(parameter_model)
      request_url_dict = request_dict.get('_url')
      if request_url_dict:
        path = '{name}'.format_map(request_url_dict)
      else:
        path = '{name}'

    query_params = request_dict.get('_query')
    if query_params:
      path = f'{path}?{urlencode(query_params)}'
    # TODO: remove the hack that pops config.
    request_dict.pop('config', None)

    http_options: Optional[types.HttpOptions] = None
    if (
        parameter_model.config is not None
        and parameter_model.config.http_options is not None
    ):
      http_options = parameter_model.config.http_options

    request_dict = _common.convert_to_dict(request_dict)
    request_dict = _common.encode_unserializable_types(request_dict)

    response = await self._api_client.async_request(
        'delete', path, request_dict, http_options
    )

  async def _list(
      self,
      *,
      parent: str,
      config: Optional[types.ListDocumentsConfigOrDict] = None,
  ) -> types.ListDocumentsResponse:
    parameter_model = types._ListDocumentsParameters(
        parent=parent,
        config=config,
    )

    request_url_dict: Optional[dict[str, str]]
    if self._api_client.vertexai:
      raise ValueError(
          'This method is only supported in the Gemini Developer client.'
      )
    else:
      request_dict = _ListDocumentsParameters_to_mldev(parameter_model)
      request_url_dict = request_dict.get('_url')
      if request_url_dict:
        path = '{parent}/documents'.format_map(request_url_dict)
      else:
        path = '{parent}/documents'

    query_params = request_dict.get('_query')
    if query_params:
      path = f'{path}?{urlencode(query_params)}'
    # TODO: remove the hack that pops config.
    request_dict.pop('config', None)

    http_options: Optional[types.HttpOptions] = None
    if (
        parameter_model.config is not None
        and parameter_model.config.http_options is not None
    ):
      http_options = parameter_model.config.http_options

    request_dict = _common.convert_to_dict(request_dict)
    request_dict = _common.encode_unserializable_types(request_dict)

    response = await self._api_client.async_request(
        'get', path, request_dict, http_options
    )

    response_dict = {} if not response.body else json.loads(response.body)

    if not self._api_client.vertexai:
      response_dict = _ListDocumentsResponse_from_mldev(response_dict)

    return_value = types.ListDocumentsResponse._from_response(
        response=response_dict, kwargs=parameter_model.model_dump()
    )

    self._api_client._verify_response(return_value)
    return return_value

  async def list(
      self,
      *,
      parent: str,
      config: Optional[types.ListDocumentsConfigOrDict] = None,
  ) -> AsyncPager[types.Document]:
    """Lists documents asynchronously.

    Args:
      parent (str): The name of the RagStore containing the Documents.
      config (ListDocumentsConfig): Optional configuration for the list request.

    Returns:
      A Pager object that contains one page of documents. When iterating over
      the pager, it automatically fetches the next page if there are more.
    Usage:
    .. code-block:: python
      async for document in await
      client.aio.documents.list(parent='rag_store_name'):
        print(f"document: {document.name} - {document.display_name}")
    """

    list_request = partial(self._list, parent=parent)
    return AsyncPager(
        'documents',
        list_request,
        await self._list(parent=parent, config=config),
        config,
    )



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


"""Unit Tests for the APIError class.

End to end tests should be in models/test_generate_content.py.
"""

import pickle

import httpx
import pytest

from ... import errors


def test_constructor_code_none_error_in_json_code_in_error():

  actual_error = errors.APIError(
      None,
      {
          'error': {
              'code': 400,
              'message': 'error message',
              'status': 'INVALID_ARGUMENT',
          }
      },
      httpx.Response(status_code=400),
  )

  assert actual_error.code == 400
  assert actual_error.message == 'error message'
  assert actual_error.status == 'INVALID_ARGUMENT'
  assert actual_error.details == {
      'error': {
          'code': 400,
          'message': 'error message',
          'status': 'INVALID_ARGUMENT',
      }
  }


def test_constructor_code_none_error_in_json_code_outside_error():

  actual_error = errors.APIError(
      None,
      {
          'code': 400,
          'error': {
              'code': 500,
              'message': 'error message',
              'status': 'INVALID_ARGUMENT',
          },
      },
      httpx.Response(status_code=400),
  )

  assert actual_error.code == 400
  assert actual_error.message == 'error message'
  assert actual_error.status == 'INVALID_ARGUMENT'
  assert actual_error.details == {
      'code': 400,
      'error': {
          'code': 500,
          'message': 'error message',
          'status': 'INVALID_ARGUMENT',
      },
  }


def test_constructor_code_not_present():

  actual_error = errors.APIError(
      None,
      {
          'error': {
              'message': 'error message',
              'status': 'INVALID_ARGUMENT',
          }
      },
      httpx.Response(status_code=400),
  )

  assert actual_error.code is None
  assert actual_error.message == 'error message'
  assert actual_error.status == 'INVALID_ARGUMENT'
  assert actual_error.details == {
      'error': {
          'message': 'error message',
          'status': 'INVALID_ARGUMENT',
      }
  }


def test_constructor_code_exist_error_in_json():

  actual_error = errors.APIError(
      400,
      {
          'error': {
              'code': 400,
              'message': 'error message',
              'status': 'INVALID_ARGUMENT',
          }
      },
      httpx.Response(status_code=400),
  )

  assert actual_error.code == 400
  assert actual_error.message == 'error message'
  assert actual_error.status == 'INVALID_ARGUMENT'
  assert actual_error.details == {
      'error': {
          'code': 400,
          'message': 'error message',
          'status': 'INVALID_ARGUMENT',
      }
  }


def test_constructor_error_not_in_json():

  actual_error = errors.APIError(
      400,
      {
          'message': 'error message',
          'status': 'INVALID_ARGUMENT',
          'code': 400,
      },
      httpx.Response(status_code=400),
  )

  assert actual_error.code == 400
  assert actual_error.message == 'error message'
  assert actual_error.status == 'INVALID_ARGUMENT'
  assert actual_error.details == {
      'message': 'error message',
      'status': 'INVALID_ARGUMENT',
      'code': 400,
  }


def test_constructor_error_in_json_status_outside_error():

  actual_error = errors.APIError(
      400,
      {
          'status': 'OUTER_INVALID_ARGUMENT_STATUS',
          'error': {
              'code': 400,
              'message': 'error message',
              'status': 'INNER_INVALID_ARGUMENT_STATUS',
          },
      },
      httpx.Response(status_code=400),
  )

  assert actual_error.code == 400
  assert actual_error.message == 'error message'
  assert actual_error.status == 'OUTER_INVALID_ARGUMENT_STATUS'
  assert actual_error.details == {
      'status': 'OUTER_INVALID_ARGUMENT_STATUS',
      'error': {
          'code': 400,
          'message': 'error message',
          'status': 'INNER_INVALID_ARGUMENT_STATUS',
      },
  }


def test_constructor_status_not_present():

  actual_error = errors.APIError(
      400,
      {
          'error': {
              'code': 400,
              'message': 'error message',
          }
      },
      httpx.Response(status_code=400),
  )

  assert actual_error.code == 400
  assert actual_error.message == 'error message'
  assert actual_error.status == None
  assert actual_error.details == {
      'error': {
          'code': 400,
          'message': 'error message',
      }
  }


def test_constructor_error_in_json_message_outside_error():

  actual_error = errors.APIError(
      400,
      {
          'message': 'OUTER_ERROR_MESSAGE',
          'error': {
              'code': 400,
              'message': 'INNER_ERROR_MESSAGE',
              'status': 'INVALID_ARGUMENT',
          },
      },
      httpx.Response(status_code=400),
  )

  assert actual_error.code == 400
  assert actual_error.message == 'OUTER_ERROR_MESSAGE'
  assert actual_error.status == 'INVALID_ARGUMENT'
  assert actual_error.details == {
      'message': 'OUTER_ERROR_MESSAGE',
      'error': {
          'code': 400,
          'message': 'INNER_ERROR_MESSAGE',
          'status': 'INVALID_ARGUMENT',
      },
  }


def test_constructor_message_not_present():

  actual_error = errors.APIError(
      400,
      {
          'error': {
              'code': 400,
              'status': 'INVALID_ARGUMENT',
          }
      },
      httpx.Response(status_code=400),
  )

  assert actual_error.code == 400
  assert actual_error.message is None
  assert actual_error.status == 'INVALID_ARGUMENT'
  assert actual_error.details == {
      'error': {
          'code': 400,
          'status': 'INVALID_ARGUMENT',
      }
  }


def test_constructor_with_websocket_connection_closed_error():
  actual_error = errors.APIError(
      1007,
      'At most one response modality can be specified in the setup request.'
      ' To enable simultaneous transcription and audio output,',
      None,
  )
  assert actual_error.code == 1007
  assert (
      actual_error.details
      == 'At most one response modality can be specified in the setup request.'
      ' To enable simultaneous transcription and audio output,'
  )
  assert actual_error.status == None
  assert actual_error.message == None


def test_raise_for_websocket_connection_closed_error():
  try:
    errors.APIError.raise_error(
        1007,
        'At most one response modality can be specified in the setup request.'
        ' To enable simultaneous transcription and audio output,',
        None,
    )
  except errors.APIError as actual_error:
    assert actual_error.code == 1007
    assert (
        actual_error.details
        == 'At most one response modality can be specified in the setup'
        ' request.'
        ' To enable simultaneous transcription and audio output,'
    )
    assert actual_error.status == None
    assert actual_error.message == None


def test_raise_for_response_code_exist_json_decoder_error():
  class FakeResponse(httpx.Response):

    def read(self) -> bytes:
      self._content = b'{"data": {"key1": "value1", "key2"}'
      return self._content

  try:
    errors.APIError.raise_for_response(
        FakeResponse(
            status_code=503,
            extensions={'reason_phrase': b'Service Unavailable'},
        )
    )
  except errors.ServerError as actual_error:
    assert actual_error.code == 503
    assert actual_error.message == '{"data": {"key1": "value1", "key2"}'
    assert actual_error.status == 'Service Unavailable'
    assert actual_error.details == {
        'message': '{"data": {"key1": "value1", "key2"}',
        'status': 'Service Unavailable',
    }


def test_raise_for_response_client_error():
  class FakeResponse(httpx.Response):

    def read(self) -> bytes:
      self._content = (
          b'{"error": {"code": 400, "message": "error message", "status":'
          b' "INVALID_ARGUMENT"}}'
      )
      return self._content

  try:
    errors.APIError.raise_for_response(FakeResponse(status_code=400))
  except errors.ClientError as actual_error:
    assert actual_error.code == 400
    assert actual_error.message == 'error message'
    assert actual_error.status == 'INVALID_ARGUMENT'
    assert actual_error.details == {
        'error': {
            'code': 400,
            'message': 'error message',
            'status': 'INVALID_ARGUMENT',
        }
    }


def test_raise_for_response_server_error():
  class FakeResponse(httpx.Response):

    def read(self) -> bytes:
      self._content = (
          b'{"error": {"code": 500, "message": "error message", "status":'
          b' "SERVER_INTERNAL ERROR"}}'
      )
      return self._content

  try:
    errors.APIError.raise_for_response(FakeResponse(status_code=500))
  except errors.ServerError as actual_error:
    assert actual_error.code == 500
    assert actual_error.message == 'error message'
    assert actual_error.status == 'SERVER_INTERNAL ERROR'
    assert actual_error.details == {
        'error': {
            'code': 500,
            'message': 'error message',
            'status': 'SERVER_INTERNAL ERROR',
        }
    }


def test_api_error_is_picklable():
  pickled_error = pickle.loads(pickle.dumps(errors.APIError(1, {})))
  assert isinstance(pickled_error, errors.APIError)


@pytest.mark.asyncio
async def test_raise_for_async_response_client_error():
  class FakeResponse(httpx.Response):

    async def aread(self) -> bytes:
      self._content = (
          b'{"error": {"code": 400, "message": "error message", "status":'
          b' "INVALID_ARGUMENT"}}'
      )
      return self._content

  try:
    await errors.APIError.raise_for_async_response(
        FakeResponse(status_code=400)
    )
  except errors.ClientError as actual_error:
    assert actual_error.code == 400
    assert actual_error.message == 'error message'
    assert actual_error.status == 'INVALID_ARGUMENT'
    assert actual_error.details == {
        'error': {
            'code': 400,
            'message': 'error message',
            'status': 'INVALID_ARGUMENT',
        }
    }


@pytest.mark.asyncio
async def test_raise_for_async_response_server_error():
  class FakeResponse(httpx.Response):

    async def aread(self) -> bytes:
      self._content = (
          b'{"error": {"code": 500, "message": "error message", "status":'
          b' "SERVER_INTERNAL ERROR"}}'
      )
      return self._content

  try:
    await errors.APIError.raise_for_async_response(
        FakeResponse(status_code=500)
    )
  except errors.ServerError as actual_error:
    assert actual_error.code == 500
    assert actual_error.message == 'error message'
    assert actual_error.status == 'SERVER_INTERNAL ERROR'
    assert actual_error.details == {
        'error': {
            'code': 500,
            'message': 'error message',
            'status': 'SERVER_INTERNAL ERROR',
        }
    }


@pytest.mark.asyncio
async def test_raise_for_async_response_code_exist_json_decoder_error():
  class FakeResponse(httpx.Response):

    async def aread(self) -> bytes:
      self._content = b'{"data": {"key1": "value1", "key2"}'
      return self._content

  try:
    await errors.APIError.raise_for_async_response(
        FakeResponse(
            status_code=503,
            extensions={'reason_phrase': b'Service Unavailable'},
        )
    )
  except errors.ServerError as actual_error:
    assert actual_error.code == 503
    assert actual_error.message == '{"data": {"key1": "value1", "key2"}'
    assert actual_error.status == 'Service Unavailable'
    assert actual_error.details == {
        'message': '{"data": {"key1": "value1", "key2"}',
        'status': 'Service Unavailable',
    }



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from __future__ import annotations

import asyncio
import functools
from typing import TypeVar, Callable, Awaitable
from typing_extensions import ParamSpec

import anyio
import sniffio
import anyio.to_thread

T_Retval = TypeVar("T_Retval")
T_ParamSpec = ParamSpec("T_ParamSpec")


async def to_thread(
    func: Callable[T_ParamSpec, T_Retval], /, *args: T_ParamSpec.args, **kwargs: T_ParamSpec.kwargs
) -> T_Retval:
    if sniffio.current_async_library() == "asyncio":
        return await asyncio.to_thread(func, *args, **kwargs)

    return await anyio.to_thread.run_sync(
        functools.partial(func, *args, **kwargs),
    )


# inspired by `asyncer`, https://github.com/tiangolo/asyncer
def asyncify(function: Callable[T_ParamSpec, T_Retval]) -> Callable[T_ParamSpec, Awaitable[T_Retval]]:
    """
    Take a blocking function and create an async one that receives the same
    positional and keyword arguments.

    Usage:

    ```python
    def blocking_func(arg1, arg2, kwarg1=None):
        # blocking code
        return result


    result = asyncify(blocking_function)(arg1, arg2, kwarg1=value1)
    ```

    ## Arguments

    `function`: a blocking regular callable (e.g. a function)

    ## Return

    An async function that takes the same positional and keyword arguments as the
    original one, that when called runs the same original function in a thread worker
    and returns the result.
    """

    async def wrapper(*args: T_ParamSpec.args, **kwargs: T_ParamSpec.kwargs) -> T_Retval:
        return await to_thread(function, *args, **kwargs)

    return wrapper



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


"""Tests for find_afc_incompatible_tool_indexes."""

from typing import Any
import pytest
from ... import types
from ..._extra_utils import find_afc_incompatible_tool_indexes

try:
  from mcp import types as mcp_types
  from mcp import ClientSession as McpClientSession
  from ..._adapters import McpToGenAiToolAdapter
except ImportError as e:
  import sys

  if sys.version_info < (3, 10):
    raise ImportError(
        'MCP Tool requires Python 3.10 or above. Please upgrade your Python'
        ' version.'
    ) from e
  else:
    raise e


def get_weather_tool(city: str) -> str:
  """Get the weather in a city."""
  return f'The weather in {city} is sunny and 100 degrees.'


class MockMcpClientSession(McpClientSession):

  def __init__(self):
    self._read_stream = None
    self._write_stream = None

  async def call_tool(self, name: str, arguments: dict[str, Any]) -> Any:
    return mcp_types.CallToolResult(
        content=[mcp_types.TextContent(type='text', text='1.01')]
    )

mcp_to_genai_tool_adapter = McpToGenAiToolAdapter(
    session=MockMcpClientSession(),
    list_tools_result=mcp_types.ListToolsResult(tools=[]),
)


def test_no_config_returns_empty_list():
    """Verifies that an empty list is returned if the input config is None.
    """
    result = find_afc_incompatible_tool_indexes(config=None)
    assert result == []

def test_config_with_no_tools_returns_empty_list():
    """Verifies an empty list is returned if the config has no 'tools' attribute.
    """
    result = find_afc_incompatible_tool_indexes(
        config=types.GenerateContentConfig()
    )
    assert result == []

def test_empty_tools_list_returns_empty_list():
    """Verifies that an empty list is returned if the 'tools' list is empty."""
    result = find_afc_incompatible_tool_indexes(
        config=types.GenerateContentConfig(tools=[])
    )

    assert result == []

def test_all_compatible_tools_returns_empty_list_with_empty_fd():
    """Verifies that an empty list is returned when all tools are compatible.

    A tool is compatible if it's not a `types.Tool` or if its
    `function_declarations` attribute is empty or None from config.
    """
    result = find_afc_incompatible_tool_indexes(
        config=types.GenerateContentConfig(
            tools=[
                types.Tool(
                    google_search_retrieval=types.GoogleSearchRetrieval()
                ),
                types.Tool(retrieval=types.Retrieval()),
                types.Tool(google_search=types.GoogleSearch()),
                types.Tool(code_execution=types.ToolCodeExecution()),
                types.Tool(google_maps=types.GoogleMaps()),
                types.Tool(url_context=types.UrlContext()),
                types.Tool(computer_use=types.ComputerUse()),
                types.Tool(code_execution=types.ToolCodeExecution()),
                types.Tool(function_declarations=[]),
                mcp_types.Tool(
                    name='get_weather',
                    description='Get the weather in a city.',
                    inputSchema={
                        'type': 'object',
                        'properties': {'location': {'type': 'string'}},
                    },
                ),
                get_weather_tool,
                mcp_to_genai_tool_adapter,
            ]
        )
    )

    assert result == []

def test_all_compatible_tools_returns_empty_list_with_none_fd():
    """Verifies that an empty list is returned when all tools are compatible.

    A tool is compatible if it's not a `types.Tool` or if its
    `function_declarations` attribute is empty or None from config.
    """
    result = find_afc_incompatible_tool_indexes(
        config=types.GenerateContentConfig(
            tools=[
                types.Tool(
                    google_search_retrieval=types.GoogleSearchRetrieval()
                ),
                types.Tool(retrieval=types.Retrieval()),
                types.Tool(google_search=types.GoogleSearch()),
                types.Tool(code_execution=types.ToolCodeExecution()),
                types.Tool(google_maps=types.GoogleMaps()),
                types.Tool(url_context=types.UrlContext()),
                types.Tool(computer_use=types.ComputerUse()),
                types.Tool(code_execution=types.ToolCodeExecution()),
                types.Tool(function_declarations=None),
                mcp_types.Tool(
                    name='get_weather',
                    description='Get the weather in a city.',
                    inputSchema={
                        'type': 'object',
                        'properties': {'location': {'type': 'string'}},
                    },
                ),
                get_weather_tool,
                mcp_to_genai_tool_adapter,
            ]
        )
    )

    assert result == []

def test_all_compatible_tools_returns_empty_list():
    """Verifies that an empty list is returned when all tools are compatible.

    A tool is compatible if it's not a `types.Tool` or if its
    `function_declarations` attribute is empty or None from config.
    """
    result = find_afc_incompatible_tool_indexes(
        config=types.GenerateContentConfig(
            tools=[
                types.Tool(
                    google_search_retrieval=types.GoogleSearchRetrieval()
                ),
                types.Tool(retrieval=types.Retrieval()),
                types.Tool(google_search=types.GoogleSearch()),
                types.Tool(code_execution=types.ToolCodeExecution()),
                types.Tool(google_maps=types.GoogleMaps()),
                types.Tool(url_context=types.UrlContext()),
                types.Tool(computer_use=types.ComputerUse()),
                types.Tool(code_execution=types.ToolCodeExecution()),
                types.Tool(function_declarations=[]),
                mcp_types.Tool(
                    name='get_weather',
                    description='Get the weather in a city.',
                    inputSchema={
                        'type': 'object',
                        'properties': {'location': {'type': 'string'}},
                    },
                ),
                get_weather_tool,
                mcp_to_genai_tool_adapter,
            ]
        )
    )

    assert result == []

def test_single_incompatible_tool():
    """Verifies that the correct index is returned for a single incompatible
    tool.
    """
    result = find_afc_incompatible_tool_indexes(
        config=types.GenerateContentConfig(
            tools=[
                types.Tool(
                    google_search_retrieval=types.GoogleSearchRetrieval()
                ),
                types.Tool(retrieval=types.Retrieval()),
                types.Tool(
                    function_declarations=[
                        types.FunctionDeclaration(name='test_function')
                    ]
                ),
                get_weather_tool,
                mcp_to_genai_tool_adapter,
            ]
        )
    )
    assert result == [2]


def test_multiple_incompatible_tools():
    """Verifies correct indexes are returned for multiple incompatible tools."""
    result = find_afc_incompatible_tool_indexes(
        config=types.GenerateContentConfig(
            tools=[
                types.Tool(
                    google_search_retrieval=types.GoogleSearchRetrieval()
                ),
                types.Tool(retrieval=types.Retrieval()),
                types.Tool(
                    function_declarations=[
                        types.FunctionDeclaration(name='test_function')
                    ]
                ),
                types.Tool(computer_use=types.ComputerUse()),
                types.Tool(code_execution=types.ToolCodeExecution()),
                types.Tool(
                    function_declarations=[
                        types.FunctionDeclaration(name='test_function_2')
                    ]
                ),
                get_weather_tool,
                mcp_to_genai_tool_adapter,
            ]
        )
    )
    assert result == [2, 5]

def test_mcp_tool_incompatible():
    """Verifies correct indexes are returned for multiple incompatible tools."""
    result = find_afc_incompatible_tool_indexes(
        config=types.GenerateContentConfig(
            tools=[
                types.Tool(
                    google_search_retrieval=types.GoogleSearchRetrieval()
                ),
                types.Tool(retrieval=types.Retrieval()),
                types.Tool(
                    function_declarations=[
                        types.FunctionDeclaration(name='test_function')
                    ]
                ),
                types.Tool(code_execution=types.ToolCodeExecution()),

                get_weather_tool,
                mcp_to_genai_tool_adapter,
                types.Tool(
                    mcp_servers=[types.McpServer(name='test_mcp_server')]
                ),
            ]
        )
    )
    assert result == [2, 6]



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import Literal, Annotated, TypeAlias

from .._utils import PropertyInfo
from .._models import BaseModel
from .text_content import TextContent
from .image_content import ImageContent

__all__ = ["FunctionResultContent", "ResultFunctionResultSubcontentList"]

ResultFunctionResultSubcontentList: TypeAlias = Annotated[
    Union[TextContent, ImageContent], PropertyInfo(discriminator="type")
]


class FunctionResultContent(BaseModel):
    """A function tool result content block."""

    call_id: str
    """Required. ID to match the ID from the function call block."""

    result: Union[List[ResultFunctionResultSubcontentList], str, object]
    """The result of the tool call."""

    type: Literal["function_result"]

    is_error: Optional[bool] = None
    """Whether the tool call resulted in an error."""

    name: Optional[str] = None
    """The name of the tool that was called."""

    signature: Optional[str] = None
    """A signature hash for backend validation."""



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._types import Base64FileInput
from .._utils import PropertyInfo
from .._models import set_pydantic_config
from .url_context_call_arguments_param import URLContextCallArgumentsParam

__all__ = ["URLContextCallContentParam"]


class URLContextCallContentParam(TypedDict, total=False):
    """URL context content."""

    id: Required[str]
    """Required. A unique ID for this specific tool call."""

    arguments: Required[URLContextCallArgumentsParam]
    """Required. The arguments to pass to the URL context."""

    type: Required[Literal["url_context_call"]]

    signature: Annotated[Union[str, Base64FileInput], PropertyInfo(format="base64")]
    """A signature hash for backend validation."""


set_pydantic_config(URLContextCallContentParam, {"arbitrary_types_allowed": True})



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the 'License');
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an 'AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""Tests for t_batch."""

import pytest
from ... import _transformers as t
from ... import client as google_genai_client_module
from ... import types


@pytest.fixture
def vertex_client():
  yield google_genai_client_module.Client(
      vertexai=True, project='test-project', location='test-location'
  )


@pytest.fixture
def mldev_client():
  yield google_genai_client_module.Client(
      vertexai=False, api_key='test-api-key'
  )


@pytest.mark.usefixtures('vertex_client', 'mldev_client')
class TestBatchJobSource:

  # Test cases for src as str
  @pytest.mark.parametrize(
      'src_str, expected_gcs_uri, expected_bigquery_uri, expected_file_name,'
      ' expected_format',
      [
          (
              'gs://bucket/path/to/data.jsonl',
              ['gs://bucket/path/to/data.jsonl'],
              None,
              None,
              'jsonl',
          ),
          (
              'bq://project.dataset.table',
              None,
              'bq://project.dataset.table',
              None,
              'bigquery',
          ),
          ('files/data.csv', None, None, 'files/data.csv', None),
      ],
  )
  def test_batch_job_source_str(
      self,
      mldev_client,
      src_str,
      expected_gcs_uri,
      expected_bigquery_uri,
      expected_file_name,
      expected_format,
  ):
    result = t.t_batch_job_source(mldev_client, src_str)
    expected = types.BatchJobSource(
        gcs_uri=expected_gcs_uri,
        bigquery_uri=expected_bigquery_uri,
        file_name=expected_file_name,
        format=expected_format,
    )
    assert result == expected

  def test_batch_job_source_str_unsupported(self, mldev_client):
    with pytest.raises(ValueError, match='Unsupported source'):
      t.t_batch_job_source(mldev_client, 'http://example.com/data')
    with pytest.raises(ValueError, match='Unsupported source'):
      t.t_batch_job_source(mldev_client, 'invalid-path')

  # Test cases for src as list
  def test_batch_job_source_list_empty(self, mldev_client):
    result = t.t_batch_job_source(mldev_client, [])
    expected = types.BatchJobSource(inlined_requests=[])
    assert result == expected

  def test_batch_job_source_list_with_items(self, mldev_client):
    inlined_data = [
        types.InlinedRequest(
            contents=[types.Content(parts=[types.Part(text='item1')])]
        ),
        types.InlinedRequest(
            contents=[types.Content(parts=[types.Part(text='item2')])]
        ),
    ]
    result = t.t_batch_job_source(mldev_client, inlined_data)
    expected = types.BatchJobSource(inlined_requests=inlined_data)
    assert result == expected

  # Test cases for src as dict
  def test_batch_job_source_dict(self, vertex_client, mldev_client):
    src_dict = {'gcs_uri': ['gs://test/file.jsonl'], 'format': 'jsonl'}
    result = t.t_batch_job_source(vertex_client, src_dict)
    expected = types.BatchJobSource(
        gcs_uri=['gs://test/file.jsonl'], format='jsonl'
    )
    assert result == expected

    src_dict_inlined = {
        'inlined_requests': [{
            'contents': [{
                'parts': [{
                    'text': 'Hello!',
                }],
                'role': 'user',
            }],
        }]
    }
    result_inlined = t.t_batch_job_source(mldev_client, src_dict_inlined)
    expected_inlined = types.BatchJobSource(inlined_requests=[{
            'contents': [{
                'parts': [{
                    'text': 'Hello!',
                }],
                'role': 'user',
            }],
        }])
    assert result_inlined == expected_inlined

  # Test cases for src as types.BatchJobSource (mocked MockBatchJobSource)

  def test_batch_job_source_mldev_valid_file_name(self, mldev_client):
    src_obj = types.BatchJobSource(file_name='files/my_data.csv')
    result = t.t_batch_job_source(mldev_client, src_obj)
    assert result is src_obj

  def test_batch_job_source_mldev_invalid_both_set(self, mldev_client):
    src_obj = types.BatchJobSource(
        inlined_requests=[types.InlinedRequest()],
        file_name='files/data.csv',
    )
    with pytest.raises(
        ValueError,
        match='`inlined_requests`, `file_name`,',
    ):
      t.t_batch_job_source(mldev_client, src_obj)

  def test_batch_job_source_mldev_invalid_neither_set(self, mldev_client):
    src_obj = types.BatchJobSource(
        gcs_uri=['gs://temp']
    )
    with pytest.raises(
        ValueError,
        match='`inlined_requests`, `file_name`,',
    ):
      t.t_batch_job_source(mldev_client, src_obj)

  # client.vertexai = True (Vertex AI)
  def test_batch_job_source_vertexai_valid_gcs(self, vertex_client):
    src_obj = types.BatchJobSource(gcs_uri=['gs://vertex-bucket/data.jsonl'])
    result = t.t_batch_job_source(vertex_client, src_obj)
    assert result is src_obj

  def test_batch_job_source_vertexai_valid_bigquery(self, vertex_client):
    src_obj = types.BatchJobSource(bigquery_uri='bq://project.dataset.table')
    result = t.t_batch_job_source(vertex_client, src_obj)
    assert result is src_obj

  def test_batch_job_source_vertexai_valid_both(self, vertex_client):
    src_obj = types.BatchJobSource(
        gcs_uri=['gs://vertex-bucket/data.jsonl'],
        bigquery_uri='bq://project.dataset.table',
    )
    with pytest.raises(ValueError, match='`gcs_uri` or `bigquery_uri`'):
      t.t_batch_job_source(vertex_client, src_obj)

  def test_batch_job_source_vertexai_invalid_neither_set(self, vertex_client):
    src_obj = types.BatchJobSource(
        file_name='files/data.csv'
    )
    with pytest.raises(ValueError, match='`gcs_uri` or `bigquery_uri`'):
      t.t_batch_job_source(vertex_client, src_obj)


class TestBatchJobDestination:

  @pytest.mark.parametrize(
      'dest_str, expected_gcs_uri, expected_bigquery_uri, expected_format',
      [
          (
              'gs://bucket/path/to/output',
              'gs://bucket/path/to/output',
              None,
              'jsonl',
          ),
          (
              'bq://project.dataset.output_table',
              None,
              'bq://project.dataset.output_table',
              'bigquery',
          ),
      ],
  )
  def test_valid_destinations(
      self, dest_str, expected_gcs_uri, expected_bigquery_uri, expected_format
  ):
    result = t.t_batch_job_destination(dest_str)
    expected = types.BatchJobDestination(
        gcs_uri=expected_gcs_uri,
        bigquery_uri=expected_bigquery_uri,
        format=expected_format,
    )
    assert result == expected

  def test_unsupported_destination(self):
    with pytest.raises(ValueError, match='Unsupported destination'):
      t.t_batch_job_destination('local/path/output.jsonl')
    with pytest.raises(ValueError, match='Unsupported destination'):
      t.t_batch_job_destination('http://some.url')


class TestBatchJobName:

  def test_mldev_valid_name(self, mldev_client):
    name = 'batches/my-job-123'
    assert t.t_batch_job_name(mldev_client, name) == 'my-job-123'

  @pytest.mark.parametrize(
      'invalid_name',
      [
          'my-job-123',
          'batches/my-job/suffix',
          'batches/',
          'batches/my-job-123/',
      ],
  )
  def test_mldev_invalid_name(self, mldev_client, invalid_name):
    with pytest.raises(ValueError, match='Invalid batch job name'):
      t.t_batch_job_name(mldev_client, invalid_name)

  def test_vertexai_valid_full_path(self, vertex_client):
    name = 'projects/123/locations/us-central1/batchPredictionJobs/my-vertex-job-456'
    assert t.t_batch_job_name(vertex_client, name) == 'my-vertex-job-456'

  def test_vertexai_valid_digit_name(self, vertex_client):
    name = '9876543210'
    assert t.t_batch_job_name(vertex_client, name) == '9876543210'

  @pytest.mark.parametrize(
      'invalid_name',
      [
          'my-vertex-job-456',
          'projects/123/locations/us-central1/job/my-vertex-job-456',
          'projects/123/locations/us-central1/batchPredictionJobs/',
          'not-a-number-or-path',
          'batchPredictionJobs/my-job',
      ],
  )
  def test_vertexai_invalid_name(self, vertex_client, invalid_name):
    with pytest.raises(ValueError, match='Invalid batch job name'):
      t.t_batch_job_name(vertex_client, invalid_name)


class TestJobState:

  @pytest.mark.parametrize(
      'input_state, expected_output_state',
      [
          ('BATCH_STATE_UNSPECIFIED', 'JOB_STATE_UNSPECIFIED'),
          ('BATCH_STATE_PENDING', 'JOB_STATE_PENDING'),
          ('BATCH_STATE_SUCCEEDED', 'JOB_STATE_SUCCEEDED'),
          ('BATCH_STATE_FAILED', 'JOB_STATE_FAILED'),
          ('BATCH_STATE_CANCELLED', 'JOB_STATE_CANCELLED'),
          ('BATCH_STATE_EXPIRED', 'JOB_STATE_EXPIRED'),
          ('BATCH_STATE_RUNNING', 'JOB_STATE_RUNNING'),
          ('BATCH_STATE_FOOBAR', 'BATCH_STATE_FOOBAR'),
      ],
  )
  def test_job_state_mapping(self, input_state, expected_output_state):
    assert t.t_job_state(input_state) == expected_output_state



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union
from typing_extensions import Annotated, TypeAlias

from .._utils import PropertyInfo
from .error_event import ErrorEvent
from .content_stop import ContentStop
from .content_delta import ContentDelta
from .content_start import ContentStart
from .interaction_start_event import InteractionStartEvent
from .interaction_status_update import InteractionStatusUpdate
from .interaction_complete_event import InteractionCompleteEvent

__all__ = ["InteractionSSEEvent"]

InteractionSSEEvent: TypeAlias = Annotated[
    Union[
        InteractionStartEvent,
        InteractionCompleteEvent,
        InteractionStatusUpdate,
        ContentStart,
        ContentDelta,
        ContentStop,
        ErrorEvent,
    ],
    PropertyInfo(discriminator="event_type"),
]



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


"""Tests for tunings.tune()."""

import time
from ... import _replay_api_client
from ... import types as genai_types
from .. import pytest_helper


test_table: list[pytest_helper.TestTableItem] = []

pytestmark = pytest_helper.setup(
    file=__file__,
    globals_for_file=globals(),
    test_method="tunings.tune",
    test_table=test_table,
)

pytest_plugins = ("pytest_asyncio",)


def test_tune_until_success(client):
  if client._api_client.vertexai:
    job = client.tunings.tune(
        base_model="gemini-2.5-flash-lite",
        training_dataset=genai_types.TuningDataset(
            gcs_uri="gs://cloud-samples-data/ai-platform/generative_ai/gemini-2_0/text/sft_train_data.jsonl",
        ),
        config=genai_types.CreateTuningJobConfig(
            epoch_count=1,
        ),
    )
  else:
    # Remove GenAI SDK test since it is deprecated:
    # https://ai.google.dev/gemini-api/docs/model-tuning
    return

  while not job.has_ended:
    # Skipping the sleep for when in replay mode.
    if client._api_client._mode not in ("replay", "auto"):
      time.sleep(300)
    job = client.tunings.get(name=job.name)

  assert job.has_ended
  assert job.has_succeeded


def test_continuous_tuning(client):
  if not client._api_client.vertexai:
    return

  job = client.tunings.tune(
      base_model="gemini-2.5-flash",
      training_dataset=genai_types.TuningDataset(
          gcs_uri="gs://cloud-samples-data/ai-platform/generative_ai/gemini-2_0/text/sft_train_data.jsonl",
      ),
      config=genai_types.CreateTuningJobConfig(
          epoch_count=1,
      ),
  )

  while not job.has_ended:
    # Skipping the sleep for when in replay mode.
    if client._api_client._mode not in ("replay", "auto"):
      time.sleep(300)
    job = client.tunings.get(name=job.name)

  assert job.has_ended
  assert job.has_succeeded

  continuous_job = client.tunings.tune(
      base_model=job.tuned_model.model,
      training_dataset=genai_types.TuningDataset(
          gcs_uri="gs://cloud-samples-data/ai-platform/generative_ai/gemini-2_0/text/sft_train_data.jsonl",
      ),
      config=genai_types.CreateTuningJobConfig(
          tuned_model_display_name="continuous tuning job",
          epoch_count=1,
      )
  )

  while not continuous_job.has_ended:
    # Skipping the sleep for when in replay mode.
    if client._api_client._mode not in ("replay", "auto"):
      time.sleep(300)
    continuous_job = client.tunings.get(name=continuous_job.name)

  assert continuous_job.has_ended
  assert continuous_job.has_succeeded





# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


"""Tests tunings.tune(), tunings.get(), tunings.cancel()."""

from .... import types as genai_types
from ... import pytest_helper


def create_get_cancel(client, parameters):
  tuning_job = client.tunings.tune(
      base_model=parameters.base_model,
      training_dataset=parameters.training_dataset,
      config=parameters.config,
  )
  tuning_job = client.tunings.get(name=tuning_job.name)
  client.tunings.cancel(name=tuning_job.name)


test_table: list[pytest_helper.TestTableItem] = [
    pytest_helper.TestTableItem(
        name="test_create_get_cancel",
        parameters=genai_types.CreateTuningJobParameters(
            base_model="gemini-2.5-flash",
            training_dataset=genai_types.TuningDataset(
                gcs_uri="gs://cloud-samples-data/ai-platform/generative_ai/gemini-2_0/text/sft_train_data.jsonl",
            ),
            config=genai_types.CreateTuningJobConfig(
                epoch_count=1,
            ),
        ),
        exception_if_mldev="not supported in Gemini API.",
    ),
]

pytestmark = pytest_helper.setup(
    file=__file__,
    globals_for_file=globals(),
    test_method="create_get_cancel",
    test_table=test_table,
)

pytest_plugins = ("pytest_asyncio",)
